@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,9 +52,11 @@ import java.util.ArrayList;
 import java.util.List;
 
 import org.apache.commons.lang.StringUtils;
+import org.apache.commons.logging.Log;
 
 import com.openexchange.office.calcengine.JCalcDocumentWrapper;
 import com.openexchange.office.tools.logging.ELogLevel;
+import com.openexchange.office.tools.logging.LogFactory;
 
 //=============================================================================
 /** seams to be some kind of over engineering ... but is useful for testing purposes.
@@ -63,35 +65,41 @@ import com.openexchange.office.tools.logging.ELogLevel;
  */
 public class CalcEngineNativeWrapper
 {
+    //-------------------------------------------------------------------------
+    private static final Log LOG = LogFactory.getJclLog(CalcEngineNativeWrapper.class);
+
     //-------------------------------------------------------------------------
     public CalcEngineNativeWrapper ()
         throws Exception
     {}
     
     //-------------------------------------------------------------------------
-    public boolean createDocument(String sHandle)
+    public synchronized boolean createDocument(String sHandle)
         throws Exception
     {
+    	LOG.trace("["+this+"] create document ...");
         return JCalcDocumentWrapper.createDocument(sHandle);
     }
     
     //-------------------------------------------------------------------------
-    public boolean destroyDocument(final String sHandle)
+    public synchronized boolean destroyDocument(final String sHandle)
         throws Exception
     {
+    	LOG.trace("["+this+"] destroy document ...");
         return JCalcDocumentWrapper.destroyDocument(sHandle);
     }
     
     //-------------------------------------------------------------------------
-    public String executeOperation(final String     sHandle   ,
-                                    final String     sOperation)
+    public synchronized String executeOperation(final String     sHandle   ,
+                                                final String     sOperation)
         throws Exception
     {
+    	LOG.trace("["+this+"] execute operation ...");
         return JCalcDocumentWrapper.executeOperation(sHandle, sOperation);
     }
 
     //-------------------------------------------------------------------------
-    public List< String > getHandleList()
+    public synchronized List< String > getHandleList()
         throws Exception
     {
     	final String         LIST_SEPERATOR = ",";
@@ -118,21 +126,21 @@ public class CalcEngineNativeWrapper
     }
 
     //-------------------------------------------------------------------------
-    public String getHandleNames()
+    public synchronized String getHandleNames()
         throws Exception
     {
         return JCalcDocumentWrapper.getAllHandles();
     }
 
     //-------------------------------------------------------------------------
-    public String getVersion()
+    public synchronized String getVersion()
         throws Exception
     {
         return JCalcDocumentWrapper.getVersion();
     }
 
     //-------------------------------------------------------------------------
-    public void setLogLevel(final ELogLevel eLevel)
+    public synchronized void setLogLevel(final ELogLevel eLevel)
         throws Exception
     {
     	final int nLevel = ELogLevel.toInt(eLevel);
