@@ -1,19 +1,19 @@
 /*
  *  Copyright 2010-2011, Plutext Pty Ltd.
- *   
+ *
  *  This file is part of docx4j.
 
-    docx4j is licensed under the Apache License, Version 2.0 (the "License"); 
-    you may not use this file except in compliance with the License. 
+    docx4j is licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
 
-    You may obtain a copy of the License at 
+    You may obtain a copy of the License at
 
-        http://www.apache.org/licenses/LICENSE-2.0 
+        http://www.apache.org/licenses/LICENSE-2.0
 
-    Unless required by applicable law or agreed to in writing, software 
-    distributed under the License is distributed on an "AS IS" BASIS, 
-    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
-    See the License for the specific language governing permissions and 
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
     limitations under the License.
 
  */
@@ -25,12 +25,9 @@ import java.util.List;
 
 import javax.xml.bind.JAXBElement;
 
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 import org.docx4j.dml.CTHyperlink;
 import org.docx4j.dml.CTNonVisualDrawingProps;
 import org.docx4j.dml.diagram.CTDataModel;
-import org.docx4j.jaxb.Context;
 import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
 import org.docx4j.openpackaging.parts.WordprocessingML.CommentsPart;
 import org.docx4j.openpackaging.parts.WordprocessingML.EndnotesPart;
@@ -45,41 +42,40 @@ import org.docx4j.utils.CompoundTraversalUtilVisitorCallback;
 import org.docx4j.utils.SingleTraversalUtilVisitorCallback;
 import org.docx4j.utils.TraversalUtilVisitor;
 import org.docx4j.wml.Body;
-import org.docx4j.wml.CTFtnEdn;
 import org.docx4j.wml.CTObject;
-import org.docx4j.wml.FldChar;
-import org.docx4j.wml.P;
-import org.docx4j.wml.Pict;
-import org.docx4j.wml.SdtContentBlock;
 import org.docx4j.wml.Comments.Comment;
+import org.docx4j.wml.FldChar;
+import org.docx4j.wml.Pict;
 import org.docx4j.wml.SdtBlock;
 import org.jvnet.jaxb2_commons.ppp.Child;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 
 /**
- * Traverse a list of JAXB objects (eg document.xml's document/body 
+ * Traverse a list of JAXB objects (eg document.xml's document/body
  * children), and do something to them.
- * 
+ *
  * This is similar to what one could do via XSLT,
  * but avoids marshalling/unmarshalling.  The downside is that
  * not everything will necessarily get traversed here
  * since visitChildren is not (yet) comprehensive.
- * 
+ *
  * This utility might be redundant if we used
- * http://code.google.com/p/jaxb-visitor/ or 
+ * http://code.google.com/p/jaxb-visitor/ or
  * https://github.com/ops4j/org.ops4j.xvisitor
  * at XJC time. Haven't tried them though.
- * 
+ *
  * See also org.docx4j.utils.SingleTraversalUtilVisitorCallback
  * and CompoundTraversalUtilVisitorCallback
- * 
+ *
  * @author jharrop, alberto
  *
  */
 public class TraversalUtil {
 
 	private static Logger log = LoggerFactory.getLogger(TraversalUtil.class);
-	
+
 	public interface Callback {
 
 		void walkJAXBElements(Object parent);
@@ -88,10 +84,10 @@ public class TraversalUtil {
 
 		/**
 		 * Visits a node in pre order (before its children have been visited).
-		 * 
+		 *
 		 * A node is visited only if all its parents have been traversed (
 		 * {@link #shouldTraverse(Object)}).</p>
-		 * 
+		 *
 		 * <p>
 		 * Implementations can have side effects.
 		 * </p>
@@ -100,31 +96,31 @@ public class TraversalUtil {
 
 		/**
 		 * Decide whether this node's children should be traversed.
-		 * 
+		 *
 		 * @return whether the children of this node should be visited
 		 */
 		boolean shouldTraverse(Object o);
-		
+
 		// TODO for next gen interface, add
 		// List<Object> applyAfterWalkChildren(Object o);
 
-	}	
+	}
 
 	public static abstract class CallbackImpl implements Callback {
-		
+
 		// Depth first
 		public void walkJAXBElements(Object parent) {
-			
+
 			List children = getChildren(parent);
 			if (children != null) {
 
 				for (Object o : children) {
-					
+
 					// if its wrapped in javax.xml.bind.JAXBElement, get its
 					// value; this is ok, provided the results of the Callback
 					// won't be marshalled
 					o = XmlUtils.unwrap(o);
-					
+
 					// workaround for broken getParent (since 3.0.0)
 					if (o instanceof Child) {
 						if (parent instanceof SdtBlock) {
@@ -132,7 +128,7 @@ public class TraversalUtil {
 								/*
 								 * getParent on eg a P in a SdtBlock should return SdtContentBlock, as
 								 * illustrated by the following code:
-								 * 
+								 *
 										SdtBlock sdtBloc = Context.getWmlObjectFactory().createSdtBlock();
 										SdtContentBlock sdtContentBloc = Context.getWmlObjectFactory().createSdtContentBlock();
 										sdtBloc.setSdtContent(sdtContentBloc);
@@ -143,9 +139,9 @@ public class TraversalUtil {
 										SdtBlock rtp = (SdtBlock)XmlUtils.unmarshalString(result, Context.jc, SdtBlock.class);
 										P rtr = (P)rtp.getSdtContent().getContent().get(0);
 										System.out.println(rtr.getParent().getClass().getName() );
-								 * 
+								 *
 								 * Similarly, P is the parent of R; the p.getContent() list is not the parent
-								 * 
+								 *
 										P p = Context.getWmlObjectFactory().createP();
 										R r = Context.getWmlObjectFactory().createR();
 										p.getContent().add(r);
@@ -159,7 +155,7 @@ public class TraversalUtil {
 							((Child)o).setParent(parent);
 						}
 					}
-					
+
 					this.apply(o);
 
 					if (this.shouldTraverse(o)) {
@@ -177,10 +173,10 @@ public class TraversalUtil {
 		/**
 		 * Visits a node in pre order (before its children have been visited).
 
-		 * 
+		 *
 		 * A node is visited only if all its parents have been traversed (
 		 * {@link #shouldTraverse(Object)}).</p>
-		 * 
+		 *
 		 * <p>
 		 * Implementations can have side effects.
 		 * </p>
@@ -189,7 +185,7 @@ public class TraversalUtil {
 
 		/**
 		 * Decide whether this node's children should be traversed.
-		 * 
+		 *
 		 * @return whether the children of this node should be visited
 		 */
 		public boolean shouldTraverse(Object o) {
@@ -210,7 +206,7 @@ public class TraversalUtil {
 
 	}
 
-	
+
 	private static List<Object> handleGraphicData(org.docx4j.dml.GraphicData graphicData) {
         // Added by Peter Buil
         List<Object> tmpArtificialList = new ArrayList<Object>();
@@ -235,8 +231,8 @@ public class TraversalUtil {
             return graphicData.getAny();
         }
     }
-	
-	 
+
+
 	/**
 	 * There can be hyperlinks references in CTNonVisualDrawingProps.
 	 * @param drawingProps
@@ -247,12 +243,12 @@ public class TraversalUtil {
           CTHyperlink docPrHyperLink = drawingProps.getHlinkClick();
           if (docPrHyperLink != null)
               artificialList.add(docPrHyperLink);
-      } 
+      }
 	}
-	
-	
+
+
 	public static List<Object> getChildrenImpl(Object o) {
-		
+
 		if (o==null) {
 			log.warn("null passed to getChildrenImpl");
 			return null;
@@ -260,7 +256,7 @@ public class TraversalUtil {
 
 		log.debug("getting children of " + o.getClass().getName() );
 		if (o instanceof org.docx4j.wml.Text) return null;
-		
+
 		// Short circuit for common elements
 		if (o instanceof List) {
 			// Handy if you have your own list of objects you wish to process
@@ -304,10 +300,10 @@ public class TraversalUtil {
                 return artificialList;
 
         } else if (o instanceof Pict) {
-			return ((Pict)o).getAnyAndAny(); // (why didn't the reflection below find this?)
-			
+			return ((Pict)o).getContent(); // (why didn't the reflection below find this?)
+
 		} else if (o instanceof org.docx4j.dml.picture.Pic) { // Post 2.7.1; untested
-			
+
 			org.docx4j.dml.picture.Pic dmlPic = ((org.docx4j.dml.picture.Pic)o);
 			if (dmlPic.getBlipFill()!=null
 					&& dmlPic.getBlipFill().getBlip()!=null) {
@@ -316,10 +312,10 @@ public class TraversalUtil {
 					artificialList.add(dmlPic.getBlipFill().getBlip());
 					return artificialList;
 			} else {
-				return null;						
-			}		
+				return null;
+			}
 		} else if (o instanceof org.docx4j.dml.CTGvmlPicture) {  // Post 2.7.1
-			
+
 			org.docx4j.dml.CTGvmlPicture dmlPic = ((org.docx4j.dml.CTGvmlPicture)o);
 			if (dmlPic.getBlipFill()!=null
 					&& dmlPic.getBlipFill().getBlip()!=null) {
@@ -328,28 +324,28 @@ public class TraversalUtil {
 					artificialList.add(dmlPic.getBlipFill().getBlip());
 					return artificialList;
 			} else {
-				return null;						
-			}		
-			
-		} else if (o instanceof org.docx4j.vml.CTShape) {				
+				return null;
+			}
+
+		} else if (o instanceof org.docx4j.vml.CTShape) {
 //			return ((org.docx4j.vml.CTShape)o).getAny();
 			List<Object> artificialList = new ArrayList<Object>();
-			for (JAXBElement<?> j : ((org.docx4j.vml.CTShape)o).getPathOrFormulasOrHandles() ) {
+			for (JAXBElement<?> j : ((org.docx4j.vml.CTShape)o).getEGShapeElements() ) {
 //				System.out.println(XmlUtils.unwrap(j).getClass().getName() );
-				artificialList.add(j);				
+				artificialList.add(j);
 			}
 			return artificialList;
 		} else if (o instanceof CTDataModel) {
 			CTDataModel dataModel = (CTDataModel)o;
 			List<Object> artificialList = new ArrayList<Object>();
-			// We're going to create a list merging two children ..			
+			// We're going to create a list merging two children ..
 			artificialList.addAll(dataModel.getPtLst().getPt());
-			artificialList.addAll(dataModel.getCxnLst().getCxn());			
+			artificialList.addAll(dataModel.getCxnLst().getCxn());
 			return artificialList;
 		} else if (o instanceof org.docx4j.dml.diagram2008.CTDrawing) {
 			return ((org.docx4j.dml.diagram2008.CTDrawing)o).getSpTree().getSpOrGrpSp();
-		} else if (o instanceof org.docx4j.vml.CTTextbox) {				
-//			return ((org.docx4j.vml.CTTextbox)o).getAny();			
+		} else if (o instanceof org.docx4j.vml.CTTextbox) {
+//			return ((org.docx4j.vml.CTTextbox)o).getAny();
 			org.docx4j.vml.CTTextbox textBox = ((org.docx4j.vml.CTTextbox)o);
 			if (textBox.getTxbxContent()==null) {
 				return null;
@@ -357,13 +353,13 @@ public class TraversalUtil {
 				return textBox.getTxbxContent().getEGBlockLevelElts();
 				// grandchildren
 			}
-//		} else if (o instanceof org.docx4j.wml.CTTxbxContent) {				
+//		} else if (o instanceof org.docx4j.wml.CTTxbxContent) {
 //			return ((org.docx4j.wml.CTTxbxContent)o).getEGBlockLevelElts();
 		} else if (o instanceof CTObject) {
-			
+
 			CTObject ctObject = (CTObject)o;
 			List<Object> artificialList = new ArrayList<Object>();
-			artificialList.addAll(ctObject.getAnyAndAny());
+			artificialList.addAll(ctObject.getContent());
 			if (ctObject.getControl()!=null) {
 				artificialList.add(ctObject.getControl() ); // CTControl
 			}
@@ -393,11 +389,11 @@ public class TraversalUtil {
 			Method[] methods = o.getClass().getDeclaredMethods();
 			for (int i = 0; i<methods.length; i++) {
 				Method m = methods[i];
-				if (m.getReturnType().getName().equals("java.util.List") ) {					
-					return (List<Object>)m.invoke(o);					
+				if (m.getReturnType().getName().equals("java.util.List") ) {
+					return (List<Object>)m.invoke(o);
 				}
 			}
-			
+
 		} catch (Exception e) {
 			// TODO Auto-generated catch block
 			e.printStackTrace();
@@ -405,7 +401,7 @@ public class TraversalUtil {
 		log.debug(".. no list member");
 		return null;
 	}
-	
+
 
 	public static void main(String[] args) throws Exception {
 
@@ -425,14 +421,14 @@ public class TraversalUtil {
 		new Callback() {
 
 			String indent = "";
-			
+
 			@Override
 			public List<Object> apply(Object o) {
-				
+
 				String text = "";
 				if (o instanceof org.docx4j.wml.Text)
 					text = ((org.docx4j.wml.Text)o).getValue();
-				
+
 				System.out.println(indent + o.getClass().getName() + "  \"" + text + "\"");
 				return null;
 			}
@@ -447,7 +443,7 @@ public class TraversalUtil {
 			public void walkJAXBElements(Object parent) {
 
 				indent += "    ";
-				
+
 				List children = getChildren(parent);
 				if (children != null) {
 
@@ -466,7 +462,7 @@ public class TraversalUtil {
 
 					}
 				}
-				
+
 				indent = indent.substring(0, indent.length()-4);
 			}
 
@@ -482,16 +478,16 @@ public class TraversalUtil {
 	}
 
 	public static void replaceChildren(Object o, List<Object> newChildren) {
-		
+
 		// Available if you need something like this.  Would be used with
 		// something like:
-		
+
 		/*
 		 * 	  public void walkJAXBElements(Object parent){
 		  // Breadth first
-				
-			  List<Object> newChildren = new ArrayList<Object>(); 
-			  
+
+			  List<Object> newChildren = new ArrayList<Object>();
+
 				List children = getChildren(parent);
 				if (children==null) {
 					log.warn("no children: " + parent.getClass().getName() );
@@ -500,48 +496,48 @@ public class TraversalUtil {
 
 						// if its wrapped in javax.xml.bind.JAXBElement, get its value
 						o = XmlUtils.unwrap(o);
-						
+
 						newChildren.addAll(this.apply(o));
 					}
-				}				
+				}
 				// Replace list, so we'll traverse all the new sdts we've just created
 				replaceChildren(parent, newChildren);
-				
-				
+
+
 				for (Object o : getChildren(parent) ) {
-					
+
 					this.apply(o);
-					
+
 					if ( this.shouldTraverse(o) ) {
 						walkJAXBElements(o);
 					}
-					
+
 				}
-				
+
 			}
 
 		 */
 
 		log.debug("Clearing " + o.getClass().getName() );
-		
+
 		if (o instanceof org.docx4j.wml.ContentAccessor) {
 
 			((org.docx4j.wml.ContentAccessor) o).getContent().clear();
 			((org.docx4j.wml.ContentAccessor) o).getContent().addAll(newChildren);
-			
+
 		} else if (o instanceof org.docx4j.wml.SdtElement) {
-			
+
 			((org.docx4j.wml.SdtElement) o).getSdtContent().getContent().clear();
 			((org.docx4j.wml.SdtElement) o).getSdtContent().getContent().addAll(newChildren);
-			
+
 		} else if (o instanceof org.docx4j.wml.CTTxbxContent) {
-			
+
 			((org.docx4j.wml.CTTxbxContent) o).getEGBlockLevelElts().clear();
-			((org.docx4j.wml.CTTxbxContent) o).getEGBlockLevelElts().addAll(newChildren);			
+			((org.docx4j.wml.CTTxbxContent) o).getEGBlockLevelElts().addAll(newChildren);
 
 		} else {
-			
-			log.warn("Don't know how to replaceChildren in " + o.getClass().getName() ); 
+
+			log.warn("Don't know how to replaceChildren in " + o.getClass().getName() );
 
 			if (o instanceof org.w3c.dom.Node) {
 				log.warn(" IGNORED " + ((org.w3c.dom.Node) o).getNodeName());
@@ -551,59 +547,59 @@ public class TraversalUtil {
 
 		}
 	}
-	
-	/** 
+
+	/**
 	 * Use this if there is only a single object type (eg just P's)
-	 * you are interested in doing something with.  
-	 * 
+	 * you are interested in doing something with.
+	 *
 	 * This method allows you to traverse just the main document
 	 * part, or also headers/footers, footnotes/endnotes, and comments
-	 * as well. 
-	 * 
+	 * as well.
+	 *
 	 * @param wmlPackage
 	 * @param bodyOnly
 	 * @param visitor
 	 */
-	public static void visit(WordprocessingMLPackage wmlPackage, 
+	public static void visit(WordprocessingMLPackage wmlPackage,
 			boolean bodyOnly, TraversalUtilVisitor visitor) {
-		
+
 		if (visitor != null) {
 			visit(wmlPackage, bodyOnly, new SingleTraversalUtilVisitorCallback(visitor));
 		}
 	}
 
-	/** 
+	/**
 	 * Use this if there is only a single object type (eg just P's)
 	 * you are interested in doing something with.
-	 * 
+	 *
 	 * This method is for traversing an arbitrary WML object (eg a table), as opposed to
 	 * eg the main document part, or a header.
-	 * 
+	 *
 	 * @param parent
 	 * @param visitor
 	 */
 	public static void visit(Object parent, TraversalUtilVisitor visitor) {
-		
+
 		if (visitor != null) {
 			visit(parent, new SingleTraversalUtilVisitorCallback(visitor));
 		}
 	}
-	
-	/** 
+
+	/**
 	 * Use this if there is more than one object type (eg Tables and Paragraphs)
 	 * you are interested in doing something with during the traversal.
-	 * 
+	 *
 	 * This method allows you to traverse just the main document
 	 * part, or also headers/footers, footnotes/endnotes, and comments
-	 * as well. 
-	 * 
+	 * as well.
+	 *
 	 * @param wmlPackage
 	 * @param bodyOnly
 	 * @param visitorList
 	 */
 	public static void visit(WordprocessingMLPackage wmlPackage,
 			boolean bodyOnly, List<TraversalUtilVisitor> visitorList) {
-		
+
 		CompoundTraversalUtilVisitorCallback callback = null;
 		if ((visitorList != null) && (!visitorList.isEmpty())) {
 			if (visitorList.size() > 1) {
@@ -615,19 +611,19 @@ public class TraversalUtil {
 		}
 	}
 
-	/** 
+	/**
 	 * Use this if there is more than one object type (eg Tables and Paragraphs)
 	 * you are interested in doing something with during the traversal.
-	 * 
+	 *
 	 * This method is for traversing an arbitrary WML object (eg a table), as opposed to
 	 * eg the main document part, or a header.
-	 * 
+	 *
 	 * @param parent
 	 * @param visitorList
 	 */
 	public static void visit(Object parent,
 			List<TraversalUtilVisitor> visitorList) {
-		
+
 		CompoundTraversalUtilVisitorCallback callback = null;
 		if ((visitorList != null) && (!visitorList.isEmpty())) {
 			if (visitorList.size() > 1) {
@@ -640,7 +636,7 @@ public class TraversalUtil {
 	}
 
 	public static void visit(Object parent, Callback callback) {
-		
+
 		if ((parent != null) && (callback != null)) {
 			callback.walkJAXBElements(parent);
 		}
@@ -648,12 +644,12 @@ public class TraversalUtil {
 
 	public static void visit(WordprocessingMLPackage wmlPackage,
 			boolean bodyOnly, Callback callback) {
-		
+
 		MainDocumentPart mainDocument = null;
 		RelationshipsPart relPart = null;
 		List<Relationship> relList = null;
 		List<Object> elementList = null;
-		
+
 		if ((wmlPackage != null) && (callback != null)) {
 			mainDocument = wmlPackage.getMainDocumentPart();
 			callback.walkJAXBElements(mainDocument.getJaxbElement().getBody());
@@ -694,37 +690,37 @@ public class TraversalUtil {
 			}
 		}
 	}
-	
+
 	// private void describeDrawing( org.docx4j.wml.Drawing d) {
-	//			
+	//
 	// log.info("In wml.Drawing" );
-	//			
+	//
 	// if ( d.getAnchorOrInline().get(0) instanceof Anchor ) {
-	//				
+	//
 	// log.info(" ENCOUNTERED w:drawing/wp:anchor " );
 	// // That's all for now...
-	//				
+	//
 	// } else if ( d.getAnchorOrInline().get(0) instanceof Inline ) {
-	//				
+	//
 	// // Extract
 	// w:drawing/wp:inline/a:graphic/a:graphicData/pic:pic/pic:blipFill/a:blip/@r:embed
-	//				
+	//
 	// Inline inline = (Inline )d.getAnchorOrInline().get(0);
-	//				
+	//
 	// Pic pic = inline.getGraphic().getGraphicData().getPic();
-	//					
+	//
 	// if (pic!=null) {
 	// log.info("image relationship: " + pic.getBlipFill().getBlip().getEmbed()
 	// );
 	// }
-	//				
-	//				
+	//
+	//
 	// } else {
-	//				
+	//
 	// log.info(" Didn't get Inline :(  How to handle " +
 	// d.getAnchorOrInline().get(0).getClass().getName() );
 	// }
-	//			
+	//
 	// }
 
 }
