@@ -491,7 +491,7 @@ public class ContentTypeManager  {
 			throws InvalidFormatException {
 		return new WebSettingsPart(new PartName(partName));
 	}
-	
+
 	public Part CreatePeoplePartObject(String partName)
 			throws InvalidFormatException {
 		return new PeoplePart(new PartName(partName));
@@ -685,21 +685,13 @@ public class ContentTypeManager  {
 		CTTypes types;
 
 		try {
-
-			Unmarshaller u = Context.jcContentTypes.createUnmarshaller();
-
-			//u.setSchema(org.docx4j.jaxb.WmlSchema.schema);
-			u.setEventHandler(new org.docx4j.jaxb.JaxbValidationEventHandler());
-
-			log.debug("unmarshalling " + this.getClass().getName() );
-
-			Object res = XmlUtils.unwrap(u.unmarshal( contentTypes ));
+			Object res = XmlUtils.unwrap(XmlUtils.unmarshal(contentTypes, Context.getJcContentTypes(), new org.docx4j.jaxb.JaxbValidationEventHandler()));
 			//types = (CTTypes)((JAXBElement)res).getValue();
 			types = (CTTypes)res;
 			//log.debug( types.getClass().getName() + " unmarshalled" );
 
 			if (log.isDebugEnabled()) {
-				XmlUtils.marshaltoString(res, true, true, Context.jcContentTypes );
+				XmlUtils.marshaltoString(res, true, true, Context.getJcContentTypes());
 			}
 
 			CTDefault defaultCT;
@@ -719,7 +711,6 @@ public class ContentTypeManager  {
 			}
 
 		} catch (Exception e ) {
-			log.error(e.getMessage(), e);
 			throw new InvalidFormatException("Bad [Content_Types].xml", e);
 		}
 
@@ -766,7 +757,7 @@ public class ContentTypeManager  {
     public void marshal(org.w3c.dom.Node node) throws JAXBException {
 
 		try {
-			Marshaller marshaller = Context.jcContentTypes.createMarshaller();
+			Marshaller marshaller = Context.getJcContentTypes().createMarshaller();
 
 			NamespacePrefixMapperUtils.setProperty(marshaller,
 					NamespacePrefixMapperUtils.getPrefixMapper() );
@@ -778,8 +769,6 @@ public class ContentTypeManager  {
 			log.info("content types marshalled \n\n" );
 
 		} catch (JAXBException e) {
-			//e.printStackTrace();
-			log.error(e.getMessage(), e);
 			throw e;
 		}
     }
@@ -787,7 +776,7 @@ public class ContentTypeManager  {
     public void marshal(java.io.OutputStream os) throws JAXBException {
 
 		try {
-			Marshaller marshaller = Context.jcContentTypes.createMarshaller();
+			Marshaller marshaller = Context.getJcContentTypes().createMarshaller();
 
 			NamespacePrefixMapperUtils.setProperty(marshaller,
 					NamespacePrefixMapperUtils.getPrefixMapper() );
@@ -796,8 +785,6 @@ public class ContentTypeManager  {
 			marshaller.marshal(buildTypes(), os);
 
 		} catch (JAXBException e) {
-			//e.printStackTrace();
-			log.error(e.getMessage(), e);
 			throw e;
 		}
 	}
@@ -887,7 +874,7 @@ public class ContentTypeManager  {
     public String toString() {
 
     	CTTypes types = buildTypes();
-    	return XmlUtils.marshaltoString(types, true, true, Context.jcContentTypes);
+    	return XmlUtils.marshaltoString(types, true, true, Context.getJcContentTypes());
 
     }
 
