@@ -31,12 +31,12 @@ public abstract class JaxbDmlPart<E>  extends JaxbXmlPartXPathAware<E> {
 
 	public JaxbDmlPart(PartName partName) throws InvalidFormatException {
 		super(partName);
-		setJAXBContext(Context.jc);
+		setJAXBContext(Context.getJc());
 	}
 
 	public JaxbDmlPart() throws InvalidFormatException {
 		super(new PartName("/xl/blagh.xml"));
-		setJAXBContext(Context.jc);
+		setJAXBContext(Context.getJc());
 	}
 
 	public static Part newPartForContentType(String contentType, String partName)
