@@ -25,6 +25,7 @@ import org.jvnet.jaxb2_commons.ppp.Child;
 
 import java.util.ArrayList;
 import java.util.List;
+
 import javax.xml.bind.JAXBElement;
 import javax.xml.bind.Unmarshaller;
 import javax.xml.bind.annotation.XmlAccessType;
@@ -35,6 +36,8 @@ import javax.xml.bind.annotation.XmlElementRefs;
 import javax.xml.bind.annotation.XmlRootElement;
 import javax.xml.bind.annotation.XmlTransient;
 import javax.xml.bind.annotation.XmlType;
+
+import org.docx4j.IndexedNodeList;
 import org.docx4j.math.CTOMath;
 import org.docx4j.math.CTOMathPara;
 import org.slf4j.Logger;
@@ -113,7 +116,7 @@ public class P implements Child, ContentAccessor
         @XmlElementRef(name = "sdt", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main", type = JAXBElement.class),
         @XmlElementRef(name = "bookmarkStart", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main", type = JAXBElement.class)
     })
-    protected List<Object> content;
+    protected IndexedNodeList<Object> content;
     @XmlAttribute(name = "rsidRPr", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main")
     protected String rsidRPr;
     @XmlAttribute(name = "rsidR", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main")
@@ -208,15 +211,16 @@ public class P implements Child, ContentAccessor
      * 
      * 
      */
-    public List<Object> getContent() {
+    @Override
+	public IndexedNodeList<Object> getContent() {
         if (content == null) {
-            content = new ArrayList<Object>();
+            content = new IndexedNodeList<Object>();
         }
         return this.content;
     }
 
     @Deprecated
-    public List<Object> getParagraphContent() {
+    public IndexedNodeList<Object> getParagraphContent() {
     	return getContent();
     }
     /**
@@ -501,7 +505,7 @@ public class P implements Child, ContentAccessor
             @XmlElementRef(name = "customXmlInsRangeEnd", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main", type = JAXBElement.class),
             @XmlElementRef(name = "fldSimple", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main", type = JAXBElement.class)
         })
-        protected List<Object> content;
+        protected IndexedNodeList<Object> content;
         @XmlAttribute(name = "tgtFrame", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main")
         protected String tgtFrame;
         @XmlAttribute(name = "tooltip", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main")
@@ -570,15 +574,16 @@ public class P implements Child, ContentAccessor
          * 
          * @since 2.7
          */
-        public List<Object> getContent() {
+        @Override
+		public IndexedNodeList<Object> getContent() {
             if (content == null) {
-                content = new ArrayList<Object>();
+                content = new IndexedNodeList<Object>();
             }
             return this.content;
         }
         
         @Deprecated
-        public List<Object> getParagraphContent() {
+        public IndexedNodeList<Object> getParagraphContent() {
         	return getContent();
         }        
 
