@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -62,10 +62,11 @@ import com.openexchange.documentation.RequestMethod;
 import com.openexchange.documentation.annotations.Action;
 import com.openexchange.documentation.annotations.Parameter;
 import com.openexchange.groupware.upload.UploadFile;
-import com.openexchange.office.tools.FileHelper;
-import com.openexchange.office.tools.FileHelper.StreamInfo;
+import com.openexchange.office.tools.DocFileHelper;
 import com.openexchange.office.tools.Resource;
 import com.openexchange.office.tools.ResourceManager;
+import com.openexchange.office.tools.StorageHelper;
+import com.openexchange.office.tools.StreamInfo;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.tools.encoding.Base64;
 import com.openexchange.tools.session.ServerSession;
@@ -129,49 +130,63 @@ public class AddFileAction extends DocumentFilterAction {
         final String fileId = request.getParameter("add_fileid");
         final String folderId = request.getParameter("add_folderid");
         byte[] data = null;
-        
+
         if ((null != fileId) && (null != folderId)) {
-        	final FileHelper fileHelper = new FileHelper(m_services, folderId, fileId);
-            final StreamInfo streamInfo = fileHelper.getDocumentStream(session);
-            final InputStream inputStream = streamInfo.documentStream;
-        	if (null != inputStream) {
-	        	try {
-					data = IOUtils.toByteArray(inputStream);
-				} catch (IOException e) {
-					LOG.error("Could not read file stream", e);
-				}
-        	}
+            final DocFileHelper fileHelper = new DocFileHelper(m_services, folderId, fileId);
+            final StorageHelper storageHelper = new StorageHelper(m_services, session, folderId);
+            final StreamInfo streamInfo = fileHelper.getDocumentStream(session, storageHelper);
+            final InputStream inputStream = streamInfo.getDocumentStream();
+            if (null != inputStream) {
+                try {
+                    data = IOUtils.toByteArray(inputStream);
+                } catch (IOException e) {
+                    LOG.error("Could not read file stream", e);
+                } finally {
+                    IOUtils.closeQuietly(inputStream);
+                    try {
+                        streamInfo.close();
+                    } catch (IOException e) {
+                        LOG.warn("Could not correctly close IDBasedFileAccess instance", e);
+                    }
+                }
+            } else {
+                try {
+                    streamInfo.close();
+                } catch (IOException e) {
+                    LOG.warn("Could not correctly close IDBasedFileAccess instance", e);
+                }
+            }
         } else {
-	        if (null != data64) {
-	
-	            final String searchPattern = "base64,";
-	            int pos = data64.indexOf(searchPattern);
-	
-	            if ((-1 != pos) && ((pos += searchPattern.length()) < (data64.length() - 1))) {
-	                data = Base64.decode(data64.substring(pos));
-	            }
-	        } else { // in IE9 data64 is not readable from request
-	            InputStream uploadStm = null;
-	
-	            try {
-	                if (request.hasUploads()) {
-	                    final UploadFile uploadFile = request.getFiles().get(0);
-	                    uploadStm = new FileInputStream(uploadFile.getTmpFile());
-	                }
-	            } catch (final Exception e) {
-	                LOG.error("Could not create file stream", e);
-	            }
-	
-	            if (null != uploadStm) {
-	                try {
-	                    data = IOUtils.toByteArray(uploadStm);
-	                } catch (IOException e) {
-	                    LOG.error("Could not read file stream", e);
-	                } finally {
-	                    IOUtils.closeQuietly(uploadStm);
-	                }
-	            }
-	        }
+            if (null != data64) {
+
+                final String searchPattern = "base64,";
+                int pos = data64.indexOf(searchPattern);
+
+                if ((-1 != pos) && ((pos += searchPattern.length()) < (data64.length() - 1))) {
+                    data = Base64.decode(data64.substring(pos));
+                }
+            } else { // in IE9 data64 is not readable from request
+                InputStream uploadStm = null;
+
+                try {
+                    if (request.hasUploads()) {
+                        final UploadFile uploadFile = request.getFiles().get(0);
+                        uploadStm = new FileInputStream(uploadFile.getTmpFile());
+                    }
+                } catch (final Exception e) {
+                    LOG.error("Could not create file stream", e);
+                }
+
+                if (null != uploadStm) {
+                    try {
+                        data = IOUtils.toByteArray(uploadStm);
+                    } catch (IOException e) {
+                        LOG.error("Could not read file stream", e);
+                    } finally {
+                        IOUtils.closeQuietly(uploadStm);
+                    }
+                }
+            }
         }
 
         if ((data != null) && (data.length > 0)) {
@@ -183,7 +198,7 @@ public class AddFileAction extends DocumentFilterAction {
 
             try {
                 String resname = null;
-                switch (FileHelper.getDocumentFormat(request.getParameter("filename"))) {
+                switch (DocFileHelper.getDocumentFormat(request.getParameter("filename"))) {
                 case DOCX: {
                     resname = "word/media/uid" + Long.toHexString(uid) + "." + extension;
                     break;
