@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,6 +51,7 @@ package com.openexchange.office.json.actions;
 
 import org.json.JSONException;
 import org.json.JSONObject;
+
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
 import com.openexchange.documentation.RequestMethod;
@@ -73,7 +74,7 @@ public class CopyDocumentAction extends DocumentFilterAction {
         @Parameter(name = "session", description = "A session ID previously obtained from the login module."),
         @Parameter(name = "id", description = "Object ID of the requested infoitem."),
         @Parameter(name = "folder_id", description = "Folder ID of the requested infoitem."),
-        @Parameter(name = "filename", optional=true, description = "If present the new filename of the requested filestore item to be set."),
+        @Parameter(name = "initial_filename", optional=true, description = "If present the new filename of the requested filestore item to be set."),
         @Parameter(name = "asTemplate", optional=true, description = "If present specifies that the file should be stored as template."),
         @Parameter(name = "outFolder_id", optional=true, description = "If present the folder ID where to write the requested infoitem, otherwise the folder of the source is used or if not possible the"),
         @Parameter(name = "uid", description = "The unique id of the client application.")
@@ -95,7 +96,7 @@ public class CopyDocumentAction extends DocumentFilterAction {
         final JSONObject requestData = new JSONObject();
 
         try {
-            requestData.put("fileName", request.getParameter("filename"));
+            requestData.put("fileName", request.getParameter("initial_filename"));
             requestData.put("asTemplate", request.getParameter("asTemplate"));
             requestData.put("targetFolder", request.getParameter("outFolder_id"));
         } catch (JSONException e) {
