@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,8 +50,8 @@ package com.openexchange.office.json.actions;
 
 import java.io.ByteArrayInputStream;
 import java.io.InputStream;
-import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.Map;
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.logging.Log;
 import org.json.JSONException;
@@ -64,7 +64,6 @@ import com.openexchange.documentconverter.Properties;
 import com.openexchange.exception.OXException;
 import com.openexchange.file.storage.DefaultFile;
 import com.openexchange.file.storage.File;
-import com.openexchange.file.storage.File.Field;
 import com.openexchange.file.storage.FileStorageFileAccess;
 import com.openexchange.file.storage.composition.IDBasedFileAccess;
 import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
@@ -80,20 +79,23 @@ import com.openexchange.office.json.tools.EmptyDocumentCache;
 import com.openexchange.office.json.tools.FilterExceptionToErrorCode;
 import com.openexchange.office.json.tools.OXDocumentHelper;
 import com.openexchange.office.json.tools.OXDocumentHelper.OfficeDocumentTypeProperties;
+import com.openexchange.office.tools.DocFileHelper;
 import com.openexchange.office.tools.DocumentFormatHelper;
 import com.openexchange.office.tools.DocumentFormatHelper.OfficeDocumentType;
+import com.openexchange.office.tools.DocumentMetaData;
 import com.openexchange.office.tools.DocumentType;
 import com.openexchange.office.tools.ErrorCode;
 import com.openexchange.office.tools.ExceptionToErrorCode;
 import com.openexchange.office.tools.FileDescriptor;
 import com.openexchange.office.tools.FileHelper;
-import com.openexchange.office.tools.FileHelper.StreamInfo;
+import com.openexchange.office.tools.FolderHelper;
 import com.openexchange.office.tools.ResourceManager;
+import com.openexchange.office.tools.StorageHelper;
+import com.openexchange.office.tools.StreamInfo;
 import com.openexchange.office.tools.UserConfigurationHelper;
 import com.openexchange.office.tools.UserConfigurationHelper.Mode;
 import com.openexchange.office.tools.htmldoc.ConfHelper;
 import com.openexchange.office.tools.htmldoc.GenericHtmlDocumentBuilder;
-import com.openexchange.office.tools.htmldoc.OperationReducer;
 import com.openexchange.office.tools.htmldoc.TextTableLimits;
 import com.openexchange.office.tools.message.MessagePropertyKey;
 import com.openexchange.office.tools.monitoring.CreateEvent;
@@ -102,7 +104,6 @@ import com.openexchange.office.tools.monitoring.ErrorType;
 import com.openexchange.office.tools.monitoring.Statistics;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.tools.session.ServerSession;
-import com.openexchange.tx.TransactionAwares;
 
 /**
  * {@link CreateDefaultDocumentAction}
@@ -270,8 +271,9 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
 
         try {
             // look into the soft cache for stored empty default document data
+            final User user = session.getUser();
             final IImporter importer = OXDocumentHelper.getImporter(m_services, officeDocumentType);
-            final EmptyDocumentCache.EmptyDocData cacheData = defaultDocCache.getDocData(module);
+            final EmptyDocumentCache.EmptyDocData cacheData = defaultDocCache.getDocData(module, DocFileHelper.mapUserLanguageToLangCode(user.getPreferredLanguage()));
             if (null != cacheData) {
                 // cache hit - provide the fastLoad empty document data
                 jsonResult = new JSONObject();
@@ -296,7 +298,6 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
 
                 if (null != importer) {
                     final DocumentProperties docProps = new DocumentProperties();
-                    final User user = session.getUser();
 
                     if (null != user) {
                         docProps.put(DocumentProperties.PROP_CREATOR, user.getDisplayName());
@@ -322,8 +323,10 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
             file.setFileName(createFileName + "." + extensionType);
             file.setFileMIMEType(mimeType);
 
-            // initialize json result object
-            jsonResult = new JSONObject();
+            if (null == jsonResult) {
+                // initialize json result object
+                jsonResult = new JSONObject();
+            }
 
             ErrorCode errorCode = ErrorCode.NO_ERROR;
             if (module.equalsIgnoreCase("text") && (null == cacheData) && (null != inputStream)) {
@@ -335,25 +338,25 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
                 inputStream = oxDocHelper.getInputStream();
 
                 // now we have to create the new document file using the input stream
-                errorCode = impl_createFileAndWriteStream(session, null, file, inputStream);
+                errorCode = FileHelper.createFileAndWriteStream(m_services, session, null, file, inputStream);
                 if (!errorCode.isError()) {
                     UserConfigurationHelper userConfHelper = new UserConfigurationHelper(m_services, session, "io.ox/office", Mode.WRITE_BACK);
                     final Boolean fastLoad = userConfHelper.getBoolean("text/documentFastLoad", true);
                     if (fastLoad != null && fastLoad) {
                         final TextTableLimits textTableLimits = ConfHelper.getTextTableLimits(userConfHelper);
-                        OperationReducer.reduceOperationCount(documentOperations);
+                        // OperationReducer.reduceOperationCount(documentOperations);
                         htmlDoc = GenericHtmlDocumentBuilder.buildHtmlDocument(documentOperations, file.getId(), file.getFolderId(), file.getVersion(), textTableLimits);
                         defaultDocCache.setDocData(module,
                             defaultDocCache.new EmptyDocData(htmlDoc,
-                                                             documentOperations.getJSONArray("operations"),
-                                                             oxDocHelper.getDocumentBuffer()));
+                                documentOperations.getJSONArray("operations"),
+                                oxDocHelper.getDocumentBuffer()));
                         jsonResult.put(MessagePropertyKey.KEY_HTMLDOCUMENT, htmlDoc);
                     }
                     jsonResult.put(MessagePropertyKey.KEY_OPERATIONS, documentOperations.getJSONArray("operations"));
                 }
             } else {
                 // now we have to create the new document file using the input stream
-                errorCode = impl_createFileAndWriteStream(session, null, file, inputStream);
+                errorCode = FileHelper.createFileAndWriteStream(m_services, session, null, file, inputStream);
                 if (!errorCode.isError() && (null != cacheData)) {
                     // set result data in case of a cache hit and the new file could be created
                     if (null != cacheData.htmlDoc) {
@@ -407,7 +410,7 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
         final boolean preserveFileName = Boolean.parseBoolean(request.getParameter("preserve_filename"));
         final boolean asTemplate = Boolean.parseBoolean(request.getParameter("as_template"));
         String folderId = request.getParameter("folder_id");
-        final String templateAccessId = FileHelper.getFileAccessId(fileId, folderId);
+        final String templateAccessId = fileId;
         String version = (null != versionStr) ? versionStr : "";
         InputStream inputStm = null;
         DocumentType eventDocumentType = DocumentType.NONE;
@@ -448,8 +451,9 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
                 loadRequest.putParameter("version", version);
                 loadRequest.putParameter("fileName", templateFileName);
 
-                final StreamInfo streamInfo = FileHelper.getDocumentStream(m_services, session, loadRequest);
-                final InputStream documentInputStm = streamInfo.documentStream;
+                final StorageHelper storageHelper = new StorageHelper(m_services, session, folderId);
+                final StreamInfo streamInfo = DocFileHelper.getDocumentStream(m_services, session, loadRequest, storageHelper);
+                final InputStream documentInputStm = streamInfo.getDocumentStream();
 
                 if (null != documentInputStm) {
                     HashMap<String, Object> jobProperties = new HashMap<String, Object>(4);
@@ -466,6 +470,17 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
 
                     inputStm = dcManager.convert(filterShortName, jobProperties, resultProperties);
                     IOUtils.closeQuietly(documentInputStm);
+                    streamInfo.close();
+
+                    if (null != inputStm) {
+                        Map<String, Object> meta = file.getMeta();
+
+                        if (null == meta) {
+                            file.setMeta(meta = new HashMap<String, Object>());
+                        }
+
+                        meta.put(DocumentMetaData.META_BINARY_CONVERTED, Boolean.TRUE);
+                    }
 
                     // set new mime type and extensionType
                     mimeType = conversionFormat.get(Properties.PROP_MIME_TYPE);
@@ -480,8 +495,9 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
                         }
                     }
                 } else {
+                    streamInfo.close();
                     // provide error information to the client side
-                    return new TemplateResult(new JSONObject().put(MessagePropertyKey.KEY_ERROR_DATA, streamInfo.errorCode), eventDocumentType);
+                    return new TemplateResult(new JSONObject().put(MessagePropertyKey.KEY_ERROR_DATA, streamInfo.getErrorCode()), eventDocumentType);
                 }
             }
 
@@ -490,12 +506,12 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
             inputStm = fileAccess.getDocument(templateAccessId, version);
         }
 
-        boolean canWriteToFolder = FileHelper.folderHasWriteAccess(folderAccess, folderId);
-        boolean canCreateFilesInFolder = FileHelper.folderHasCreateAccess(folderAccess, folderId);
+        boolean canWriteToFolder = FolderHelper.folderHasWriteAccess(folderAccess, folderId);
+        boolean canCreateFilesInFolder = FolderHelper.folderHasCreateAccess(folderAccess, folderId);
         if ((!canWriteToFolder || !canCreateFilesInFolder) && !preserveFileName) {
             // We cannot create files/write to the folder which contains our source template document (Edit as New).
             // Create the document in the user's default folder instead.
-            folderId = FileHelper.getUserFolderId(m_services.getService(FolderService.class), session);
+            folderId = DocFileHelper.getUserFolderId(m_services.getService(FolderService.class), session);
         }
 
         // create a new file store item with default content
@@ -517,7 +533,7 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
 
         try {
             if (null != inputStm) {
-                errorCode = impl_createFileAndWriteStream(session, fileAccess, file, inputStm);
+                errorCode = FileHelper.createFileAndWriteStream(m_services, session, fileAccess, file, inputStm);
 
                 if (!errorCode.isError()) {
                     // set the file descriptor
@@ -545,59 +561,4 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
 
         return result;
     }
-
-    /**
-     * Creates a OX Drive file and writes the provided input stream to it.
-     *
-     * @param session
-     *  The session of the user who requested to create a new file.
-     *
-     * @param fileAccess
-     *  The file access instance to be used by this method. Can be null
-     *  where the method creates it's own file access.
-     *
-     * @param file
-     *  The new file instance, which is correctly initialized.
-     *
-     * @param inputStream
-     *  The input stream of the document to be written. Attention: The stream
-     *  is not closed by the method. So the caller is responsible to close
-     *  the stream.
-     *
-     * @return
-     *  The error code of the operation which is ErrorCode.NO_ERROR on success
-     *  or any other code in case of an error.
-     */
-    private ErrorCode impl_createFileAndWriteStream(final ServerSession session, final IDBasedFileAccess fileAccess, final File file, final InputStream inputStream) {
-        IDBasedFileAccess myFileAccess = fileAccess;
-        if (null == myFileAccess) {
-            final IDBasedFileAccessFactory fileFactory = m_services.getService(IDBasedFileAccessFactory.class);
-            myFileAccess = fileFactory.createAccess(session);
-        }
-        ErrorCode errorCode = ErrorCode.NO_ERROR;
-        boolean rollback = true;
-
-        try {
-            myFileAccess.startTransaction();
-            rollback = true;
-            myFileAccess.saveDocument(file, inputStream, FileStorageFileAccess.DISTANT_FUTURE, new ArrayList<Field>());
-            myFileAccess.commit();
-            rollback = false;
-        } catch (final Exception e) {
-            errorCode = ErrorCode.GENERAL_PERMISSION_CREATE_MISSING_ERROR;
-            if (e instanceof OXException) {
-                errorCode = ExceptionToErrorCode.map((OXException)e, errorCode, false);
-            }
-            LOG.error(errorCode.getDescription(), e);
-        } finally {
-            // Roll-back (if needed) and finish
-            if (rollback) {
-                TransactionAwares.rollbackSafe(myFileAccess);
-            }
-
-            TransactionAwares.finishSafe(myFileAccess);
-        }
-
-        return errorCode;
-    }
 }
