@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,10 +51,13 @@ package com.openexchange.office.json.actions;
 import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.HashMap;
+
 import org.apache.commons.io.IOUtils;
+import org.apache.commons.lang.StringUtils;
 import org.apache.commons.logging.Log;
 import org.json.JSONException;
 import org.json.JSONObject;
+
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
 import com.openexchange.documentation.RequestMethod;
@@ -70,19 +73,26 @@ import com.openexchange.file.storage.File.Field;
 import com.openexchange.file.storage.FileStorageFileAccess;
 import com.openexchange.file.storage.composition.IDBasedFileAccess;
 import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
+import com.openexchange.file.storage.composition.IDBasedFolderAccess;
+import com.openexchange.file.storage.composition.IDBasedFolderAccessFactory;
 import com.openexchange.folderstorage.FolderService;
+import com.openexchange.groupware.infostore.InfostoreExceptionCodes;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.IImporter;
+import com.openexchange.office.json.tools.DynamicFields;
+import com.openexchange.office.tools.DocFileHelper;
 import com.openexchange.office.tools.DocumentFormatHelper;
 import com.openexchange.office.tools.DocumentType;
 import com.openexchange.office.tools.ErrorCode;
 import com.openexchange.office.tools.ExceptionToErrorCode;
 import com.openexchange.office.tools.FileDescriptor;
 import com.openexchange.office.tools.FileHelper;
-import com.openexchange.office.tools.FileHelper.StreamInfo;
+import com.openexchange.office.tools.FolderHelper;
 import com.openexchange.office.tools.LocalFileMappingManager;
 import com.openexchange.office.tools.MimeTypeHelper;
+import com.openexchange.office.tools.StorageHelper;
+import com.openexchange.office.tools.StreamInfo;
 import com.openexchange.office.tools.message.MessagePropertyKey;
 import com.openexchange.office.tools.monitoring.CreateEvent;
 import com.openexchange.office.tools.monitoring.DocumentEvent;
@@ -95,6 +105,7 @@ import com.openexchange.tx.TransactionAwares;
 public class CreateFromTemplateAction extends DocumentFilterAction {
 
     private static final Log LOG = com.openexchange.log.Log.loggerFor(CreateFromTemplateAction.class);
+    private static final String UNNAMED = "unnamed";
 
     private LocalFileMappingManager m_localFileMapper = null;
 
@@ -119,6 +130,7 @@ public class CreateFromTemplateAction extends DocumentFilterAction {
         if ((null != request) && (null != session)) {
             final IDBasedFileAccessFactory fileFactory = m_services.getService(IDBasedFileAccessFactory.class);
             final IDBasedFileAccess fileAccess = (null != fileFactory) ? fileFactory.createAccess(session) : null;
+
             DocumentType eventDocumentType = DocumentType.NONE;
             InputStream inputStm = null;
             boolean rollback = false;
@@ -127,7 +139,18 @@ public class CreateFromTemplateAction extends DocumentFilterAction {
                 final File file = new DefaultFile();
                 final String fileId = request.getParameter("id");
                 final String srcFolderId = request.getParameter("folder_id");
-                final String folderId = FileHelper.getUserFolderId(m_services.getService(FolderService.class), session);
+
+                String folderId = request.getParameter("initial_folderId");
+                if (StringUtils.isNotBlank(folderId)) {
+                    final IDBasedFolderAccessFactory folderFactory = m_services.getService(IDBasedFolderAccessFactory.class);
+                    final IDBasedFolderAccess folderAccess = (null != folderFactory) ? folderFactory.createAccess(session) : null;
+                    if (!FolderHelper.folderHasCreateAccess(folderAccess, folderId)) {
+                        folderId = null;
+                    }
+                }
+                if (StringUtils.isBlank(folderId)) {
+                    folderId = DocFileHelper.getUserFolderId(m_services.getService(FolderService.class), session);
+                }
 
                 if ((null != fileId) && (null != srcFolderId) && (null != fileAccess) && (null != folderId)) {
                     // Create document based on template file. It could be that we have to
@@ -167,19 +190,23 @@ public class CreateFromTemplateAction extends DocumentFilterAction {
                             ErrorCode errorCode = ErrorCode.NO_ERROR;
 
                             if ((null != dcManager) && dcManager.hasFeature(Feature.DOCUMENTCONVERTER)) {
+                                StreamInfo streamInfo = null;
                                 InputStream documentInputStm = null;
+
                                 if (bLocalFile) {
                                     // local file can directly read from the local file system
-                                    documentInputStm = FileHelper.getDocumentStreamFromLocalFile(fullFilePath);
+                                    documentInputStm = DocFileHelper.getDocumentStreamFromLocalFile(fullFilePath);
                                 } else {
                                     // OX Drive file must be loaded using Files API
                                     AJAXRequestData loadRequest = new AJAXRequestData();
                                     loadRequest.putParameter("id", fileId);
+                                    loadRequest.putParameter("folder_id", folderId);
                                     loadRequest.putParameter("version", version);
                                     loadRequest.putParameter("fileName", templateFileName);
-                                    final StreamInfo streamInfo = FileHelper.getDocumentStream(m_services, session, loadRequest);
-                                    documentInputStm = streamInfo.documentStream;
-                                    errorCode = streamInfo.errorCode;
+                                    final StorageHelper storageHelper = new StorageHelper(m_services, session, folderId);
+                                    streamInfo = DocFileHelper.getDocumentStream(m_services, session, loadRequest, storageHelper);
+                                    documentInputStm = streamInfo.getDocumentStream();
+                                    errorCode = streamInfo.getErrorCode();
                                 }
 
                                 if (null != documentInputStm) {
@@ -197,6 +224,10 @@ public class CreateFromTemplateAction extends DocumentFilterAction {
                                     inputStm = dcManager.convert(filterShortName, jobProperties, resultProperties);
                                     IOUtils.closeQuietly(documentInputStm);
 
+                                    if (null != streamInfo) {
+                                        streamInfo.close();
+                                    }
+
                                     // set new mime type and extensionType
                                     mimeType = conversionFormat.get(Properties.PROP_MIME_TYPE);
                                     extensionType = conversionFormat.get(Properties.PROP_INPUT_TYPE);
@@ -205,6 +236,10 @@ public class CreateFromTemplateAction extends DocumentFilterAction {
                                     documentEvent = new DocumentEvent(eventDocumentType, ErrorType.CREATE);
                                     Statistics.handleDocumentEvent(documentEvent);
 
+                                    if (null != streamInfo) {
+                                        streamInfo.close();
+                                    }
+
                                     return new AJAXRequestResult(createErrorObject(errorCode));
                                 }
                             }
@@ -214,7 +249,7 @@ public class CreateFromTemplateAction extends DocumentFilterAction {
                             // we have to set the correct target document format (created from the template)
                             HashMap<String, String> targetFormat = DocumentFormatHelper.getTemplateTargetFormatInfo(mimeType, extensionType);
                             if (bLocalFile) {
-                                inputStm = FileHelper.getDocumentStreamFromLocalFile(fullFilePath);
+                                inputStm = DocFileHelper.getDocumentStreamFromLocalFile(fullFilePath);
                             } else {
                                 inputStm = fileAccess.getDocument(fileId, version);
                             }
@@ -231,12 +266,12 @@ public class CreateFromTemplateAction extends DocumentFilterAction {
                         file.setFolderId(folderId);
 
                         final String initialFileName = request.getParameter("initial_filename");
-                        final String createFileName = ((null == initialFileName) || (initialFileName.length() < 1)) ? "unnamed" : initialFileName;
+                        final String createFileName = ((null == initialFileName) || (initialFileName.length() < 1)) ? UNNAMED : initialFileName;
                         file.setFileName(createFileName + "." + extensionType);
                         file.setFileMIMEType(mimeType);
 
                         if (null != inputStm) {
-                            final IImporter importer = DocumentFilterHelper.getImporter(m_services, FileHelper.getDocumentFormat("." + extensionType));
+                            final IImporter importer = DocumentFilterHelper.getImporter(m_services, DocFileHelper.getDocumentFormat("." + extensionType));
                             if(importer!=null) {
                                 try {
                                     final DocumentProperties documentProperties = new DocumentProperties();
@@ -258,14 +293,48 @@ public class CreateFromTemplateAction extends DocumentFilterAction {
                         }
 
                         if (null != inputStm) {
+                            final JSONObject result = new JSONObject(2);
+
+                            try
+                            {
+                                inputStm = DynamicFields.insertOptionalFields(inputStm, extensionType, request, result, session, m_services, m_localFileMapper);
+                            }
+                            catch (Exception e)
+                            {
+                                ErrorCode errorCode = ErrorCode.GENERAL_UNKNOWN_ERROR;
+                                LOG.error("Handling optional fields is broken " + errorCode.getDescription(), e);
+                                try
+                                {
+                                    requestResult = new AJAXRequestResult(createErrorObject(errorCode));
+                                    documentEvent = new DocumentEvent(eventDocumentType, ErrorType.CREATE);
+                                }
+                                catch (final JSONException je)
+                                {
+                                    LOG.warn("Couldn't create JSON object while creating new document from template");
+                                }
+                            }
+
                             fileAccess.startTransaction();
                             rollback = true;
-                            fileAccess.saveDocument(file, inputStm, FileStorageFileAccess.DISTANT_FUTURE, new ArrayList<Field>());
+                            try
+                            {
+                                LOG.trace("RT connection: [CreateFromTemplate] saveDocument using meta data: " + ((null != file) ? file.toString() : "null"));
+                                fileAccess.saveDocument(file, inputStm, FileStorageFileAccess.DISTANT_FUTURE, new ArrayList<Field>());
+                            }
+                            catch (OXException e)
+                            {
+                                //if 'only' validation of the name fails, we try it again with 'unnamend'
+                                if (e.getCode() == InfostoreExceptionCodes.VALIDATION_FAILED_CHARACTERS.getNumber() || e.getCode() == InfostoreExceptionCodes.VALIDATION_FAILED_SLASH.getNumber()){
+                                    file.setFileName(UNNAMED + "." + extensionType);
+                                    LOG.trace("RT connection: [CreateFromTemplate] saveDocument using meta data: " + ((null != file) ? file.toString() : "null"));
+                                    fileAccess.saveDocument(file, inputStm, FileStorageFileAccess.DISTANT_FUTURE, new ArrayList<Field>());
+                                } else {
+                                    throw e;
+                                }
+                            }
                             fileAccess.commit();
                             rollback = false;
 
-                            final JSONObject result = new JSONObject(2);
-
                             // set the file descriptor as a separate object
                             result.put(MessagePropertyKey.KEY_FILEDESCRIPTOR, FileDescriptor.createJSONObject(file, null));
                             // we need to provide if the file has been converted or not
