@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,26 +52,40 @@ import java.io.FilenameFilter;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.HashMap;
+import java.util.LinkedHashSet;
+import java.util.Set;
+
 import org.apache.commons.logging.Log;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+
 import com.openexchange.exception.OXException;
 import com.openexchange.file.storage.File;
+import com.openexchange.file.storage.FileStorageExceptionCodes;
+import com.openexchange.file.storage.FileStorageFolder;
 import com.openexchange.file.storage.composition.IDBasedFileAccess;
 import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
+import com.openexchange.file.storage.composition.IDBasedFolderAccess;
+import com.openexchange.file.storage.composition.IDBasedFolderAccessFactory;
 import com.openexchange.folderstorage.FolderService;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.results.TimedResult;
 import com.openexchange.office.tools.ApplicationType;
 import com.openexchange.office.tools.ConfigurationHelper;
+import com.openexchange.office.tools.DocFileHelper;
 import com.openexchange.office.tools.DocumentType;
 import com.openexchange.office.tools.ErrorCode;
 import com.openexchange.office.tools.FileHelper;
+import com.openexchange.office.tools.FolderHelper;
 import com.openexchange.office.tools.LocalFileMappingManager;
 import com.openexchange.office.tools.MimeTypeHelper;
+import com.openexchange.office.tools.UserConfigurationHelper;
+import com.openexchange.office.tools.UserConfigurationHelper.Mode;
 import com.openexchange.server.ServiceLookup;
+import com.openexchange.session.Session;
 import com.openexchange.tools.iterator.SearchIterator;
 import com.openexchange.tools.session.ServerSession;
 
@@ -79,7 +93,7 @@ public class TemplateFilesScanner {
 
     private static final Log LOG = com.openexchange.log.Log.loggerFor(TemplateFilesScanner.class);
 
-    private static boolean DISABLE_ODF_SPREADSHEET = true;
+    private static boolean DISABLE_ODF_SPREADSHEET = false;
     private static boolean DISABLE_BINARY_MS_FORMATS = true;
 
     private ServiceLookup m_services = null;
@@ -140,13 +154,13 @@ public class TemplateFilesScanner {
 
         public ExtensionFilter(String[] acceptedExtensions, String[]...more) {
             if (null != acceptedExtensions) {
-	            for (String ext : acceptedExtensions) {
-	                m_acceptedExtensions.put(ext, true);
-	            }
-	            for (String[] strArray : more) {
+                for (String ext : acceptedExtensions) {
+                    m_acceptedExtensions.put(ext, true);
+                }
+                for (String[] strArray : more) {
                     for (String ext : strArray) {
-	                    m_acceptedExtensions.put(ext, true);
-	                }
+                        m_acceptedExtensions.put(ext, true);
+                    }
                 }
             }
         }
@@ -160,8 +174,8 @@ public class TemplateFilesScanner {
     }
 
     public TemplateFilesScanner(ServiceLookup services, LocalFileMappingManager localFileMapper) {
-    	m_services = services;
-    	m_localFileMapper = localFileMapper;
+        m_services = services;
+        m_localFileMapper = localFileMapper;
         m_extensionFilter = new ExtensionFilter(MimeTypeHelper.EXCEL_TEMPLATE_EXTENSIONS,
             MimeTypeHelper.WORD_TEMPLATE_EXTENSIONS,
             MimeTypeHelper.POWERPOINT_TEMPLATE_EXTENSIONS,
@@ -173,14 +187,20 @@ public class TemplateFilesScanner {
     /**
      *
      * @return
+     * @throws OXException 
      */
-    public ErrorCode searchForTemplates(ServerSession session, String type, final JSONArray templates) {
+    public ErrorCode searchForTemplates(ServerSession session, String type, final JSONArray templates) throws OXException {
         ErrorCode errorCode = ErrorCode.NO_ERROR;
 
         if ((null != session) && (null != type) && (type.length() > 0)) {
             DocumentType searchFilter = DocumentType.NONE;
             IDBasedFileAccess fileAccess = null;
+            IDBasedFolderAccess folderAccess = null;
 
+            // We want to measure the time needed to find the templates
+            // including to check for folders that were deleted (moved to
+            // the trash folder).
+            long time1 = System.currentTimeMillis();
             // Map type to DocumenntType to restrict search filter - other values means all
             if (ApplicationType.APP_TEXT_STRING.equalsIgnoreCase(type)) {
                 searchFilter = DocumentType.TEXT;
@@ -191,19 +211,58 @@ public class TemplateFilesScanner {
             }
 
             final IDBasedFileAccessFactory fileFactory = m_services.getService(IDBasedFileAccessFactory.class);
-            final String folderId = FileHelper.getUserFolderId(m_services.getService(FolderService.class), session);
+            final IDBasedFolderAccessFactory folderFactory = m_services.getService(IDBasedFolderAccessFactory.class);
+            final String userFolderId = DocFileHelper.getUserFolderId(m_services.getService(FolderService.class), session);
             final User user = session.getUser();
             final String fallbackTemplatePath = "/opt/open-xchange/templates/documents";
             String userLangCode = "en"; // fallback if no language is set
 
+            // create file/folder access
             fileAccess = fileFactory.createAccess(session);
+            folderAccess = folderFactory.createAccess(session);
 
             if (null != user) {
-                userLangCode = FileHelper.mapUserLanguageToLangCode(user.getPreferredLanguage());
+                userLangCode = DocFileHelper.mapUserLanguageToLangCode(user.getPreferredLanguage());
             }
-
+            
             // scan the user folder and collect available document templates
-            errorCode = scanUserFolderForTemplates(fileAccess, templates, folderId, searchFilter);
+            ErrorCode userErrorCode = scanUserFolderForTemplates(fileAccess, templates, userFolderId, searchFilter);
+            if (ErrorCode.NO_ERROR == errorCode) {
+                errorCode = userErrorCode;
+            }
+                        
+            Set<String> templateFolders = new LinkedHashSet<String>();
+            //LinkedHashSet keeps the put order
+            
+            getUserData(session.getUserId(), session, "io.ox/office", "UserTemplateFolders", templateFolders);
+            getUserData(session.getContext().getMailadmin(), session, "io.ox/office", "ContextTemplateFolders", templateFolders);
+                       
+            if (null != templateFolders) {
+                ErrorCode err = null;
+                for (String folderId: templateFolders){
+                    try{
+                        // check if a user configured template folder is in the trash folder
+                        final String parentFolderId = folderAccess.getFolder(folderId).getParentId();
+                        final FileStorageFolder folder = folderAccess.getFolder(parentFolderId);
+
+                        if (!FolderHelper.isTrashFolder(folder)) {
+                            err = scanUserFolderForTemplates(fileAccess, templates, folderId, searchFilter);
+                            if (ErrorCode.NO_ERROR != err && ErrorCode.NO_ERROR == errorCode) {
+                                errorCode = err;
+                            }
+                        }
+                    } catch (OXException e) { 
+                        // moved try catch inside the loop, because context admin could set template folder from external (for ex.: Dropbox),
+                        // where only he has access to                        
+                        if (e.getCode() == FileStorageExceptionCodes.ACCOUNT_NOT_FOUND.ordinal()) {
+                            LOG.warn("Error user template paths cannot be checked to be inside the trash folder, probably user has no access to this folder in this context", e);
+                        } else {
+                            LOG.error("Error user template paths cannot be checked to be inside the trash folder", e);    
+                            break;
+                        }
+                    }
+                }
+            }
 
             // next step: we try to scan the global template path and
             // collect the files there.
@@ -214,11 +273,46 @@ public class TemplateFilesScanner {
             } else {
                 LOG.error("Error global template path is not set");
             }
+
+            // log time on info level
+            long time2 = System.currentTimeMillis();
+            LOG.info("Scan template folders including delete folder detection: " + (time2 - time1) + "ms");
         }
 
         return errorCode;
     }
 
+    /**
+     * 
+     * with help of the UserConfigurationHelper, userdata is read.
+     * whose userdata is decided by userId & contextId
+     * 
+     * id & key defines the data which is put to target-set
+     * 
+     * @param userId
+     * @param contextId
+     * @param id
+     * @param key
+     * @param target
+     * @throws OXException
+     */
+    private void getUserData(int userId, Session realSession, String id, String key, Collection<String> target) throws OXException{
+        Session session = new FakeSession(userId, realSession);
+        UserConfigurationHelper userConfHelper = new UserConfigurationHelper(m_services, session, id, Mode.WRITE_BACK);
+        final JSONArray array = userConfHelper.getJSONArray(key);
+        if (null == array){
+            return;
+        }
+        try{
+            for (int i = 0; i < array.length(); i++){
+                target.add(array.getString(i));
+            }
+        }
+        catch (JSONException e){
+            throw new OXException(e);
+        }
+    }
+
     /**
      * Collects the files within the specified folder while checking that no
      * duplicate files are added.
@@ -265,20 +359,20 @@ public class TemplateFilesScanner {
                                 // use optional file extension filter to filter
                                 // out unwanted files.
                                 if ((null == extFilter) || ((null != extFilter) && extFilter.accept(file, fileName))) {
-	                                String fileMimeType = MimeTypeHelper.getMimeTypeFromTemplateExtension(ext);
-	                                long lastModified = file.lastModified();
+                                    String fileMimeType = MimeTypeHelper.getMimeTypeFromTemplateExtension(ext);
+                                    long lastModified = file.lastModified();
 
-	                                if (DISABLE_ODF_SPREADSHEET && (MimeTypeHelper.MIMETYPE_VND_OASIS_OPENDOCUMENT_SPREADSHEET_TEMPLATE.equalsIgnoreCase(fileMimeType))) {
-	                                    // filter out odf spreadsheet
-	                                    addFile = false;
-	                                }
-	                                if (addFile && DISABLE_BINARY_MS_FORMATS) {
-	                                    addFile = !isBinaryMSTemplateMimeType(fileMimeType);
-	                                }
+                                    if (DISABLE_ODF_SPREADSHEET && (MimeTypeHelper.MIMETYPE_VND_OASIS_OPENDOCUMENT_SPREADSHEET_TEMPLATE.equalsIgnoreCase(fileMimeType))) {
+                                        // filter out odf spreadsheet
+                                        addFile = false;
+                                    }
+                                    if (addFile && DISABLE_BINARY_MS_FORMATS) {
+                                        addFile = !isBinaryMSTemplateMimeType(fileMimeType);
+                                    }
 
-	                                if (addFile) {
-	                                    putFileData(templates, true, file.getCanonicalPath(), "template:local", file.getName(), file.getName(), "global", type, lastModified, fileMimeType);
-	                                }
+                                    if (addFile) {
+                                        putFileData(templates, true, file.getCanonicalPath(), "template:local", file.getName(), file.getName(), "global", type, lastModified, fileMimeType);
+                                    }
                                 }
                             } catch (JSONException e) {
                                 // just ignore
@@ -377,6 +471,12 @@ public class TemplateFilesScanner {
                         errorCode = collectTemplateFiles(langFoldersFound[0], templates, duplicates, type, extFilter);
                     }
                 }
+
+                // Finally, check if there are some common templates
+                langFoldersFound = folder.listFiles(new LanguageFolderFilter("common", null));
+                if (langFoldersFound.length > 0) {
+                    errorCode = collectTemplateFiles(langFoldersFound[0], templates, duplicates, type, extFilter);
+                }
             }
         }
 
