@@ -37,6 +37,7 @@ import org.odftoolkit.odfdom.dom.OdfSchemaDocument.OdfXMLFile;
 import org.odftoolkit.odfdom.pkg.OdfPackage;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import org.xml.sax.SAXException;
 
 /**
  * This is a prototype of a new high level view text document. Methods might be
@@ -57,6 +58,7 @@ public class OdfOperationDocument {
     static final String MAX_TABLE_COLUMNS = "maxTableColumns";
     static final String MAX_TABLE_ROWS = "maxTableRows";
     static final String MAX_TABLE_CELLS = "maxTableCells";
+    static final String MAX_SHEETS = "maxSheets";
 
     /**
      * Only being used when a certain test (my workbench test, which is not part
@@ -70,6 +72,7 @@ public class OdfOperationDocument {
     private int mMaxTableColumnCount;
     private int mMaxTableRowCount;
     private int mMaxTableCellCount;
+    private int mMaxSheetCount;
     private boolean isMetadataUpdated = false;
 
     /**
@@ -169,6 +172,9 @@ public class OdfOperationDocument {
             if (configuration.containsKey(MAX_TABLE_CELLS)) {
                 mMaxTableCellCount = (Integer) configuration.get(MAX_TABLE_CELLS);
             }
+            if (configuration.containsKey(MAX_SHEETS)) {
+                mMaxSheetCount = (Integer) configuration.get(MAX_SHEETS);
+            }
         }
     }
 
@@ -184,6 +190,10 @@ public class OdfOperationDocument {
         return mMaxTableCellCount;
     }
 
+    public int getMaxSheetCount() {
+        return mMaxSheetCount;
+    }
+
     /**
      * Receives the a map with new resources for the Document
      *
@@ -199,7 +209,7 @@ public class OdfOperationDocument {
      * @return the operations as JSON
      */
     // ToDo OX - JSONObject is to be considered..
-    public JSONObject getOperations() {
+    public JSONObject getOperations() throws SAXException {
         JSONObject ops = mDocument.getOperations();
         if (ops != null && ops.length() > 0) {
             LOG.debug("\n\n*** ALL OPERATIONS:\n{0}", mDocument.getOperations().toString());
