@@ -50,13 +50,14 @@ public class Table<T> extends Component {
     /**
      * The maximal number of rows being generated. Unspecified by ODF but commonly used. Counting starts with 0.
      */
-    public static final Integer MAX_ROW_NUMBER = 1048575;
+    public static final Integer MAX_ROW_NUMBER = 1048576;
 
     /**
      * The maximal number of columns being generated. Unspecified by ODF but commonly used. Counting starts with 0.
      */
-    public static final Integer MAX_COLUMN_NUMBER = 16383;
-
+    public static final Integer MAX_COLUMN_NUMBER_CALC = 1024; // 2^10 There is only a flag in Calc that was changed for the OX backend having Excel Size.
+    public static final Integer MAX_COLUMN_NUMBER_EXCEL = 16384; // 2^16
+    
     /**
      * A multiple components can be represented by a single XML element
      *
@@ -157,20 +158,8 @@ public class Table<T> extends Component {
      * @param attrs the formatting attributes
      */
     public void formatColumn(int columnStartNo, int columnEndNo, JSONObject attrs) {
-        int rowEndNo = -1;
         // in ODF cell and character attributes are on a default style, that is only applied in an application to the cells existing in XML
-        if (attrs.has("cell") || attrs.has("cell")) {
-            JSONObject cellProps = attrs.optJSONObject("cell");
-            if (cellProps != null && cellProps.length() > 0) {
-                rowEndNo = MAX_ROW_NUMBER;
-            } else {
-                JSONObject characterProps = attrs.optJSONObject("character");
-                if (characterProps != null && characterProps.length() > 0) {
-                    rowEndNo = MAX_ROW_NUMBER;
-                }
-            }
-        }
-        this.modifyRange(columnStartNo, columnEndNo, 0, rowEndNo, TableChanges.newFormatation(getTableTraversal(), attrs, columnStartNo, columnEndNo, 0, rowEndNo));
+        this.modifyRange(columnStartNo, columnEndNo, 0, -1, TableChanges.newFormatation(getTableTraversal(), attrs, columnStartNo, columnEndNo, 0, -1));
     }
 
     /**
@@ -187,11 +176,11 @@ public class Table<T> extends Component {
         if (attrs.has("cell") || attrs.has("cell")) {
             JSONObject cellProps = attrs.optJSONObject("cell");
             if (cellProps != null && cellProps.length() > 0) {
-                columnEndNo = MAX_COLUMN_NUMBER;
+                columnEndNo = MAX_COLUMN_NUMBER_EXCEL - 1;
             } else {
                 JSONObject characterProps = attrs.optJSONObject("character");
                 if (characterProps != null && characterProps.length() > 0) {
-                    columnEndNo = MAX_COLUMN_NUMBER;
+                    columnEndNo = MAX_COLUMN_NUMBER_EXCEL - 1;
                 }
             }
         }
@@ -240,7 +229,7 @@ public class Table<T> extends Component {
     public static void deleteColumns(TableTableElement tableElement, int columnStartNo, int columnEndNo) {
         if (tableElement != null) {
             Table table = ((Table) tableElement.getComponent());
-            table.modifyRange(columnStartNo, columnEndNo, 0, MAX_ROW_NUMBER, TableChanges.newColumnDeletion(table.getTableTraversal(), columnStartNo, columnEndNo, 0, MAX_ROW_NUMBER));
+            table.modifyRange(columnStartNo, columnEndNo, 0, MAX_ROW_NUMBER - 1, TableChanges.newColumnDeletion(table.getTableTraversal(), columnStartNo, columnEndNo, 0, MAX_ROW_NUMBER - 1));
         }
     }
 
