@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -54,6 +54,7 @@ import java.io.InputStream;
 
 import org.apache.commons.logging.Log;
 import org.json.JSONObject;
+import org.xml.sax.SAXException;
 
 import com.openexchange.log.LogFactory;
 import com.openexchange.office.DocumentProperties;
@@ -97,15 +98,15 @@ public class Importer implements IImporter {
             try {
 
                 odfdomDocument = new OdfOperationDoc(session, services, inputDocumentStm, null, documentProperties);
-            	// Get uniqueDocumentID when existing otherwise set one..            
+            	// Get uniqueDocumentID when existing otherwise set one..
         		String existingDocumentStatusID = odfdomDocument.readStatusID();
         		if(existingDocumentStatusID != null && !existingDocumentStatusID.isEmpty()){
-        			// return the existing ODF statusID back to the system 
-        			documentProperties.put(DocumentProperties.PROP_UNIQUE_DOCUMENT_IDENTIFIER, existingDocumentStatusID);       			
-        		}	            	
-            	result = odfdomDocument.getOperations();            
-        	
-            	
+        			// return the existing ODF statusID back to the system
+        			documentProperties.put(DocumentProperties.PROP_UNIQUE_DOCUMENT_IDENTIFIER, existingDocumentStatusID);
+        		}
+            	result = odfdomDocument.getOperations();
+
+
                 /*
                  * document size is no longer needed, the editor is calculating loading time on its own // add documentSize element to
                  * result if (null != result) { final long documentSize = odfdomDocument.getContentSize(); result.put("documentSize",
@@ -113,8 +114,8 @@ public class Importer implements IImporter {
                  */
             } catch (Throwable e) {
                 LOG.error("ODF Filter could not load the given document:", e);
-                if (e instanceof FilterException) {
-                    throw (FilterException) e;
+                if (e instanceof SAXException) {
+                    throw new FilterException("Too many sheets", FilterException.ErrorCode.COMPLEXITY_TOO_HIGH);
                 } else if (e instanceof java.util.zip.ZipException && e.getMessage().contains(
                     "only DEFLATED entries can have EXT descriptor")) {
                     throw new FilterException(e, ErrorCode.UNSUPPORTED_ENCRYPTION_USED);
