@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -68,6 +68,7 @@ import org.docx4j.wml.UnderlineEnumeration;
 import org.json.JSONException;
 import org.json.JSONObject;
 
+import com.openexchange.office.FilterException;
 import com.openexchange.office.ooxml.docx.OperationDocument;
 import com.openexchange.office.ooxml.tools.Commons;
 import com.openexchange.office.ooxml.tools.ThemeFonts;
@@ -75,7 +76,7 @@ import com.openexchange.office.ooxml.tools.ThemeFonts;
 public class Character {
 
     public static void applyCharacterProperties(OperationDocument operationDocument, JSONObject characterProperties, RPrBase rPr )
-        throws JSONException {
+        throws FilterException, JSONException {
 
         if(characterProperties==null)
             return;
