@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -63,7 +63,6 @@ import org.docx4j.wml.Color;
 import org.docx4j.wml.Jc;
 import org.docx4j.wml.JcEnumeration;
 import org.docx4j.wml.ObjectFactory;
-import org.docx4j.wml.PPr;
 import org.docx4j.wml.PPrBase;
 import org.docx4j.wml.PPrBase.Ind;
 import org.docx4j.wml.PPrBase.OutlineLvl;
@@ -76,6 +75,7 @@ import org.json.JSONException;
 import org.json.JSONObject;
 
 import com.openexchange.log.LogFactory;
+import com.openexchange.office.FilterException;
 import com.openexchange.office.ooxml.docx.OperationDocument;
 import com.openexchange.office.ooxml.tools.Commons;
 
@@ -83,8 +83,8 @@ public class Paragraph {
 
     protected final static Log log = LogFactory.getLog(Paragraph.class);
 
-    public static void applyParagraphProperties(OperationDocument operationDocument, JSONObject paragraphProperties, PPr pPr)
-        throws JSONException {
+    public static void applyParagraphProperties(OperationDocument operationDocument, JSONObject paragraphProperties, PPrBase pPr)
+        throws FilterException, JSONException {
 
         if(paragraphProperties==null)
             return;
@@ -385,13 +385,13 @@ public class Paragraph {
         }
     }
 
-    public static JSONObject createParagraphProperties(PPr paragraphProperties)
+    public static JSONObject createParagraphProperties(PPrBase paragraphProperties)
         throws JSONException {
 
         JSONObject jsonParagraphProperties = null;
         if(paragraphProperties!=null) {
             jsonParagraphProperties = new JSONObject();
-            
+
             final BooleanDefaultTrue pageBreakBefore = paragraphProperties.getPageBreakBefore();
             if(pageBreakBefore!=null) {
             	jsonParagraphProperties.put("pageBreakBefore", pageBreakBefore.isVal());
