@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,13 +50,16 @@
 package com.openexchange.office.ooxml.pptx;
 
 import java.io.InputStream;
+
 import org.docx4j.dml.Theme;
-import org.docx4j.openpackaging.exceptions.Docx4JException;
+import org.docx4j.openpackaging.packages.OpcPackage;
 import org.docx4j.openpackaging.packages.PresentationMLPackage;
+import org.docx4j.openpackaging.parts.Part;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
 import org.json.JSONTokener;
+
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.FilterException.ErrorCode;
@@ -66,7 +69,7 @@ import com.openexchange.session.Session;
 
 public class OperationDocument extends com.openexchange.office.ooxml.OperationDocument {
 
-    final private PresentationMLPackage opcPackage;
+    private PresentationMLPackage opcPackage;
 
     public OperationDocument(Session session, ServiceLookup _services, InputStream _inputDocumentStream, ResourceManager _resourceManager, DocumentProperties _documentProperties)
         throws FilterException {
@@ -75,15 +78,9 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         try {
             opcPackage = (PresentationMLPackage)PresentationMLPackage.load(_inputDocumentStream);
         }
-        catch(Docx4JException e) {
-            ErrorCode errorCode = ErrorCode.CRITICAL_ERROR;
-            if(e.getCause() instanceof java.security.InvalidKeyException) {
-                errorCode = ErrorCode.WRONG_PASSWORD;
-            }
-            else if(e.getCause() instanceof org.apache.poi.EncryptedDocumentException) {
-                errorCode = ErrorCode.UNSUPPORTED_ENCRYPTION_USED;
-            }
-            throw new FilterException(e, errorCode);
+        catch(Throwable e) {
+            removeMemoryListener();
+            throw getRethrowException(e);
         }
     }
 
@@ -95,8 +92,9 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         try {
             opcPackage = PresentationMLPackage.createPackage();
         }
-        catch(Docx4JException e) {
-            throw new FilterException(e, ErrorCode.CRITICAL_ERROR);
+        catch(Throwable e) {
+            removeMemoryListener();
+            throw getRethrowException(e);
         }
     }
 
@@ -183,6 +181,15 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         return opcPackage;
     }
 
+    @Override
+    public void setPackage(OpcPackage p) {
+       opcPackage = (PresentationMLPackage)p;
+    }
+
+    public Part getContextPart() {
+    	return contextPart!=null ? contextPart : getPackage().getMainPresentationPart();
+    }
+
     @Override
     public Theme getTheme() {
         return null;
