@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -57,13 +57,15 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipInputStream;
+
 import javax.xml.bind.JAXBElement;
 import javax.xml.bind.JAXBException;
+
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.logging.Log;
 import org.docx4j.dml.Theme;
-import org.docx4j.openpackaging.exceptions.Docx4JException;
 import org.docx4j.openpackaging.exceptions.InvalidFormatException;
+import org.docx4j.openpackaging.packages.OpcPackage;
 import org.docx4j.openpackaging.packages.SpreadsheetMLPackage;
 import org.docx4j.openpackaging.parts.Part;
 import org.docx4j.openpackaging.parts.PartName;
@@ -96,15 +98,20 @@ import org.xlsx4j.sml.IDataValidations;
 import org.xlsx4j.sml.Row;
 import org.xlsx4j.sml.Sheet;
 import org.xlsx4j.sml.SheetData;
+import org.xlsx4j.sml.SheetView;
+import org.xlsx4j.sml.SheetViews;
+import org.xlsx4j.sml.Sheets;
 import org.xlsx4j.sml.Workbook;
 import org.xlsx4j.sml.Worksheet;
+
 import com.openexchange.log.LogFactory;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.FilterException.ErrorCode;
-import com.openexchange.office.PreviewData;
+import com.openexchange.office.ooxml.tools.AttrsMode;
 import com.openexchange.office.ooxml.xlsx.operations.ApplyOperationHelper;
 import com.openexchange.office.ooxml.xlsx.operations.CreateOperationHelper;
+import com.openexchange.office.ooxml.xlsx.tools.AutoFilterHelper;
 import com.openexchange.office.ooxml.xlsx.tools.SheetUtils;
 import com.openexchange.office.ooxml.xlsx.tools.ValidationUtils;
 import com.openexchange.office.tools.ConfigurationHelper;
@@ -114,10 +121,12 @@ import com.openexchange.session.Session;
 
 public class OperationDocument extends com.openexchange.office.ooxml.OperationDocument {
 
-    final private SpreadsheetMLPackage opcPackage;
+    final private int FACTOR_FOR_REQUIRED_HEAPSPACE = 32;
+    private SpreadsheetMLPackage opcPackage;
     protected CTStylesheet stylesheet = null;
     protected CTSst sharedStrings = null;
-    private ApplyOperationHelper applyOperationHelper;
+    private ApplyOperationHelper applyOperationHelper = null;
+    private CreateOperationHelper _createOperationHelper = null;
     private Double defaultFontSize = null;
 
     protected static Log log = LogFactory.getLog(OperationDocument.class);
@@ -129,9 +138,6 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         try {
 
         	inputDocumentStream = checkDocumentComplexity(_inputDocumentStream, _documentProperties.optLong(DocumentProperties.PROP_MAX_MEMORY_USAGE, -1));
-
-        	org.docx4j.jaxb.Context.setFreeHeapSizeLimit(_documentProperties.optLong(DocumentProperties.PROP_MIN_FREE_HEAPSPACE, 75000000));
-
             opcPackage = (SpreadsheetMLPackage)SpreadsheetMLPackage.load(inputDocumentStream);
 
             // styles
@@ -151,15 +157,9 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                 sheet.setName(initialSheetName);
             }
         }
-        catch(final Docx4JException e) {
-            ErrorCode errorCode = ErrorCode.CRITICAL_ERROR;
-            if(e.getCause() instanceof java.security.InvalidKeyException) {
-                errorCode = ErrorCode.WRONG_PASSWORD;
-            }
-            else if(e.getCause() instanceof org.apache.poi.EncryptedDocumentException) {
-                errorCode = ErrorCode.UNSUPPORTED_ENCRYPTION_USED;
-            }
-            throw new FilterException(e, errorCode);
+        catch(Throwable e) {
+            removeMemoryListener();
+            throw getRethrowException(e);
         }
     }
 
@@ -191,11 +191,16 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
 
             getStylesheet(true);
         }
-        catch (final Exception e) {
-            throw new FilterException(e, ErrorCode.CRITICAL_ERROR);
+        catch (Throwable e) {
+            removeMemoryListener();
+            throw getRethrowException(e);
         }
     }
 
+    public Part getContextPart() {
+    	return contextPart!=null ? contextPart : getPackage().getWorkbookPart();
+    }
+
     public CTStylesheet getStylesheet(boolean createIfMissing) {
 
         if(stylesheet==null&&createIfMissing) {
@@ -285,7 +290,7 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         return String.valueOf(si.size()-1);
     }
 
-    public List<IDataValidations> getDataValidations(Worksheet worksheet) {
+    public static List<IDataValidations> getDataValidations(Worksheet worksheet) {
         final List<IDataValidations> dataValidations = new ArrayList<IDataValidations>();
 
         final CTDataValidations validations = worksheet.getDataValidations();
@@ -321,6 +326,15 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         return applyOperationHelper;
     }
 
+    public CreateOperationHelper getCreateOperationHelper()
+    	throws FilterException {
+    	if(_createOperationHelper==null) {
+    		_createOperationHelper = new CreateOperationHelper(this, new JSONArray());
+    	}
+    	_createOperationHelper.setOperationsArray(new JSONArray());
+    	return _createOperationHelper;
+    }
+
     @Override
     public void applyOperations(String applyOperations) throws FilterException {
     	if (applyOperations != null){
@@ -335,13 +349,13 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                     if (opName.equals("setCellContents"))
                         applyOperationHelper.setCellContents(op.getInt("sheet"), op.getJSONArray("start"), op.getJSONArray("contents"));
                     else if(opName.equals("fillCellRange"))
-                        applyOperationHelper.fillCellRange(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), op.opt("value"), op.opt("result"), op.optJSONObject("attrs"), op.optInt("shared", -1), op.optJSONArray("ref"));
+                        applyOperationHelper.fillCellRange(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), op.opt("value"), op.opt("result"), op.optJSONObject("attrs"), AttrsMode.getEnum(op.optString("attrsMode", null)), op.optInt("shared", -1), op.optJSONArray("ref"));
                     else if(opName.equals("clearCellRange"))
                         applyOperationHelper.clearCellRange(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), true, true);
                     else if (opName.equals("setRowAttributes"))
-                        applyOperationHelper.setRowAttributes(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1), op.getJSONObject("attrs"));
+                        applyOperationHelper.setRowAttributes(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1), op.getJSONObject("attrs"), AttrsMode.getEnum(op.optString("attrsMode", null)));
                     else if (opName.equals("setColumnAttributes"))
-                        applyOperationHelper.setColumnAttributes(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1), op.getJSONObject("attrs"));
+                        applyOperationHelper.setColumnAttributes(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1), op.getJSONObject("attrs"), AttrsMode.getEnum(op.optString("attrsMode", null)));
                     else if (opName.equals("insertRows"))
                         applyOperationHelper.insertRows(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1));
                     else if (opName.equals("deleteRows"))
@@ -381,7 +395,7 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                     else if (opName.equals("deleteChartDataSeries"))
                         applyOperationHelper.deleteChartDataSeries(op.getJSONArray("start"), op.getInt("series"));
                     else if (opName.equals("insertName") || opName.equals("changeName"))
-                        applyOperationHelper.insertName(op.optLong("sheet", -1), op.getString("exprName"), op.getString("formula"));
+                        applyOperationHelper.insertName(op.optLong("sheet", -1), op.getString("exprName"), op.getString("formula"), null);
                     else if (opName.equals("deleteName"))
                         applyOperationHelper.deleteName(op.optLong("sheet"), op.getString("exprName"));
                     else if (opName.equals("setChartAxisAttributes"))
@@ -400,6 +414,14 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                     	ValidationUtils.deleteValidation(this, op.getInt("sheet"), op.getInt("index"));
                     else if (opName.equals("setDocumentAttributes"))
                     	applyOperationHelper.setDocumentAttributes(op.getJSONObject("attrs"));
+                    else if (opName.equals("insertTable"))
+                        AutoFilterHelper.insertTable(applyOperationHelper, op.getInt("sheet"), op.optString("table", null), op.getJSONArray("start"), op.getJSONArray("end"), op.optJSONObject("attrs"));
+                    else if (opName.equals("changeTable"))
+                        AutoFilterHelper.changeTable(applyOperationHelper, op.getInt("sheet"), op.optString("table", null), op.getJSONArray("start"), op.getJSONArray("end"), op.optJSONObject("attrs"));
+                    else if (opName.equals("deleteTable"))
+                        AutoFilterHelper.deleteTable(applyOperationHelper, op.getInt("sheet"), op.optString("table", null));
+                    else if (opName.equals("changeTableColumn"))
+                        AutoFilterHelper.changeTableColumn(applyOperationHelper, op.getInt("sheet"), op.optString("table", null), op.getLong("col"), op.getJSONObject("attrs"));
                     else if (opName.equals("createError"))
                         throw new FilterException("createError operation detected: " + opName, ErrorCode.UNSUPPORTED_OPERATION_USED);
                     else
@@ -426,6 +448,25 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                         workbookRelationshipsPart.removeRelationship(relationship);
                     }
                 }
+
+                // removing tabSelected from other worksheets
+                final Sheets sheets = workbook.getSheets();
+                final List<Sheet> sheetList = sheets.getSheet();
+                for(int sheetIndex = 0; sheetIndex < sheetList.size(); sheetIndex++) {
+                    final Sheet sheet = sheetList.get(sheetIndex);
+                    final Part part = getSheetPart(getPackage(), sheet);
+                    final String sheetType = getSheetType(part);
+                    if(sheetType.equals("worksheet")) {
+                        final Worksheet worksheet = ((WorksheetPart)part).getJaxbElement();
+                        final SheetViews sheetViews = worksheet.getSheetViews();
+                        if(sheetViews!=null) {
+                            final List<SheetView> sheetViewList = sheetViews.getSheetView();
+                            for(SheetView sheetView:sheetViewList) {
+                                sheetView.setTabSelected(null);
+                            }
+                        }
+                    }
+                }
             }
             catch(final Exception e) {
                 String message = e.getMessage();
@@ -474,22 +515,8 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
     public JSONObject getOperations()
         throws Exception {
 
-        final CreateOperationHelper createOperationHelper = new CreateOperationHelper(this, new JSONArray());
-        final Object previewObj = getDocumentProperties().get(DocumentProperties.PROP_PREVIEWDATA);
-        if((previewObj!=null) && (previewObj instanceof PreviewData)){
-            final PreviewData previewData = (PreviewData)previewObj;
-
-        	int activeSheetIndex = createOperationHelper.createPreviewOperations();
-        	final JSONObject previewOperations = new JSONObject();
-        	previewOperations.put("operations", createOperationHelper.getOperationsArray());
-        	previewData.setPreviewOperations(previewOperations);
-            previewData.setActiveSheetIndex(activeSheetIndex);
-        	createOperationHelper.setOperationsArray(new JSONArray());
-            createOperationHelper.createOutstandingOperations(activeSheetIndex);
-        }
-        else {
-        	createOperationHelper.createOperations();
-        }
+        final CreateOperationHelper createOperationHelper = getCreateOperationHelper();
+    	createOperationHelper.createOperations();
 
 /*
         Writer writer = new FileWriter(new File("/tmp/jsonout.txt"));
@@ -504,19 +531,52 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         return aOperations;
     }
 
+    public JSONObject getPreviewOperations()
+        throws Exception {
+
+        final CreateOperationHelper createOperationHelper = getCreateOperationHelper();
+    	createOperationHelper.createPreviewOperations(getDocumentProperties());
+        final JSONObject aOperations = new JSONObject();
+        aOperations.put("operations", createOperationHelper.getOperationsArray());
+        return aOperations;
+    }
+
+    public JSONObject getOutstandingOperations()
+        throws Exception {
+
+        final CreateOperationHelper createOperationHelper = getCreateOperationHelper();
+        final int activeSheetIndex = (Integer)getDocumentProperties().get(DocumentProperties.PROP_SPREADHSEET_ACTIVE_SHEET_INDEX);
+        createOperationHelper.createOutstandingOperations(activeSheetIndex);
+        final JSONObject aOperations = new JSONObject();
+        aOperations.put("operations", createOperationHelper.getOperationsArray());
+        return aOperations;
+    }
+
     @Override
     public SpreadsheetMLPackage getPackage() {
         return opcPackage;
     }
 
     @Override
-    public Theme getTheme() {
-        try {
-            return ((ThemePart)getPackage().getParts().get(new PartName("/xl/theme/theme1.xml"))).getJaxbElement();
-        }
-        catch (final Exception e) {
-            return null;
-        }
+    public void setPackage(OpcPackage p) {
+       opcPackage = (SpreadsheetMLPackage)p;
+    }
+
+    @Override
+    public Theme getTheme()
+    	throws FilterException {
+
+    	try {
+    		final PartName themePartName = new PartName("/xl/theme/theme1.xml");
+	    	ThemePart themePart = (ThemePart)getPackage().getParts().get(themePartName);
+	    	if(themePart==null) {
+	    		themePart = getDefaultThemePart(themePartName);
+	    	}
+	    	return themePart.getJaxbElement();
+    	}
+    	catch(InvalidFormatException e) {
+    		throw new FilterException("xlsx filter, could not create default theme", ErrorCode.CRITICAL_ERROR, e);
+    	}
     }
 
     public double getDefaultFontSize() {
@@ -552,8 +612,8 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
             return 8.7d;
         }
         final Double defaultColWidth = sheetFormatPr.getDefaultColWidth();
-        if(defaultColWidth!=null&&defaultColWidth.doubleValue()!=0) {
-            return defaultColWidth.doubleValue() + 0.7d;
+        if(defaultColWidth!=null) {
+            return defaultColWidth.doubleValue();
         }
         return sheetFormatPr.getBaseColWidth() + 0.7d;
     }
@@ -569,7 +629,7 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
 
     	InputStream returnValue = null;
 
-    	int maxCellCount  = ConfigurationHelper.getIntegerOfficeConfigurationValue(getServiceLookup(), getSession(), "//spreadsheet/maxCells", 500000);
+    	final int maxCellCount  = ConfigurationHelper.getIntegerOfficeConfigurationValue(getServiceLookup(), getSession(), "//spreadsheet/maxCells", 500000);
         if(maxCellCount<=0) {
         	return _inputStream;
         }
@@ -609,8 +669,8 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
 			    }
 			}
             if(maxMemoryUsage>0) {
-                if(xmlFileSize*70>maxMemoryUsage) {
-                    throw new FilterException("", ErrorCode.MEMORY_USAGE_TOO_HIGH, (long)(xmlFileSize*70));
+                if(xmlFileSize*FACTOR_FOR_REQUIRED_HEAPSPACE>maxMemoryUsage) {
+                    throw new FilterException("", ErrorCode.MEMORY_USAGE_TOO_HIGH, (long)(xmlFileSize*FACTOR_FOR_REQUIRED_HEAPSPACE));
                 }
             }
 			returnValue = new ByteArrayInputStream(bytes);
@@ -627,6 +687,36 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         return returnValue;
     }
 
+    public static Part getSheetPart(SpreadsheetMLPackage spreadsheetPackage, final Sheet sheet) {
+        final RelationshipsPart relationshipPart = spreadsheetPackage.getWorkbookPart().getRelationshipsPart();
+        return relationshipPart.getPart(sheet.getId());
+    }
+
+    public static String getSheetType(final Part part)
+        throws FilterException {
+
+        String sheetType = null;
+        if(part.getRelationshipType().equals("http://schemas.openxmlformats.org/officeDocument/2006/relationships/worksheet")) {
+            sheetType = "worksheet";
+        }
+        else if(part.getRelationshipType().equals("http://schemas.microsoft.com/office/2006/relationships/xlMacrosheet")) {
+            sheetType = "macrosheet";
+        }
+        else if(part.getRelationshipType().equals("http://schemas.microsoft.com/office/2006/relationships/xlIntlMacrosheet")) {
+            sheetType = "macrosheet";
+        }
+        else if(part.getRelationshipType().equals("http://schemas.openxmlformats.org/officeDocument/2006/relationships/dialogsheet")) {
+            sheetType = "dialogsheet";
+        }
+        else if(part.getRelationshipType().equals("http://schemas.openxmlformats.org/officeDocument/2006/relationships/chartsheet")) {
+            sheetType = "chartsheet";
+        }
+        if(sheetType==null) {
+            throw new FilterException("xlsx import: unknown sheettype detected.", ErrorCode.CRITICAL_ERROR);
+        }
+        return sheetType;
+    }
+
     private CTStylesheet createStylesheet() throws JAXBException {
 
         final String defaultStylesheet = "<styleSheet xmlns=\"http://schemas.openxmlformats.org/spreadsheetml/2006/main\"" +
