@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -57,7 +57,6 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
-import java.util.Set;
 
 import javax.xml.bind.JAXBException;
 
@@ -88,6 +87,7 @@ import org.docx4j.openpackaging.parts.DrawingML.Drawing;
 import org.docx4j.openpackaging.parts.DrawingML.JaxbDmlPart;
 import org.docx4j.openpackaging.parts.SpreadsheetML.WorkbookPart;
 import org.docx4j.openpackaging.parts.SpreadsheetML.WorksheetPart;
+import org.docx4j.openpackaging.parts.relationships.Namespaces;
 import org.docx4j.openpackaging.parts.relationships.RelationshipsPart;
 import org.docx4j.openpackaging.parts.relationships.RelationshipsPart.AddPartBehaviour;
 import org.docx4j.relationships.Relationship;
@@ -98,12 +98,14 @@ import org.json.JSONObject;
 import org.w3c.dom.Document;
 import org.xlsx4j.jaxb.Context;
 import org.xlsx4j.sml.BookViews;
+import org.xlsx4j.sml.CTAutoFilter;
 import org.xlsx4j.sml.CTBookView;
 import org.xlsx4j.sml.CTCellFormula;
 import org.xlsx4j.sml.CTCellStyle;
 import org.xlsx4j.sml.CTCellStyleXfs;
 import org.xlsx4j.sml.CTCellStyles;
 import org.xlsx4j.sml.CTDefinedName;
+import org.xlsx4j.sml.CTFilterColumn;
 import org.xlsx4j.sml.CTMergeCell;
 import org.xlsx4j.sml.CTMergeCells;
 import org.xlsx4j.sml.CTSheetFormatPr;
@@ -129,7 +131,9 @@ import com.openexchange.log.LogFactory;
 import com.openexchange.office.ooxml.drawingml.DMLChartSpace;
 import com.openexchange.office.ooxml.drawingml.DMLGraphic;
 import com.openexchange.office.ooxml.drawingml.DMLHelper;
+import com.openexchange.office.ooxml.tools.AttrsMode;
 import com.openexchange.office.ooxml.xlsx.OperationDocument;
+import com.openexchange.office.ooxml.xlsx.tools.AutoFilterHelper;
 import com.openexchange.office.ooxml.xlsx.tools.CellUtils;
 import com.openexchange.office.ooxml.xlsx.tools.ChartUtils;
 import com.openexchange.office.ooxml.xlsx.tools.ChartUtils.ColRow;
@@ -240,7 +244,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         return (WorksheetPart)relationshipPart.getPart(sheet.getId());
     }
 
-    private Worksheet getWorksheet(int sheetIndex) {
+    public Worksheet getWorksheet(int sheetIndex) {
         return getWorksheetPart(sheetIndex).getJaxbElement();
     }
 
@@ -269,13 +273,15 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 
                 final JSONObject cellObject = cellArray.getJSONObject(j);
 
-                Object value = cellObject.opt("value");
-                Object result = cellObject.opt("result");
-                if(value != null || result != null)
+                final Object value = cellObject.opt("value");
+                final Object result = cellObject.opt("result");
+                if(value!=null || result!=null) {
                 	setCellContent(cell, value, result , null, null);
-                JSONObject attrs = cellObject.optJSONObject("attrs");
-                if(attrs != null) {
-                	CellUtils.applyCellProperties(operationDocument, worksheetPart, attrs, cell, getOperationDocument().getStylesheet(true), null);
+                }
+                final JSONObject attrs = cellObject.optJSONObject("attrs");
+                final AttrsMode attrsMode = AttrsMode.getEnum(cellObject.optString("attrsMode", null));
+                if(attrs!=null||attrsMode!=AttrsMode.EXTEND) {
+                	CellUtils.applyCellPropertiesToCell(operationDocument, worksheetPart, sheetData, attrs, attrsMode, row, cell, getOperationDocument().getStylesheet(true), null);
                 }
                 x++;
             }
@@ -283,37 +289,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         }
     }
 
-    private static boolean isClearAttributes(JSONObject optAttrs) {
-        if(optAttrs!=null) {
-            if(optAttrs==JSONObject.NULL) {
-                return true;
-            }
-            final JSONObject cellAttr = optAttrs.optJSONObject("cell");
-            final JSONObject characterAttr = optAttrs.optJSONObject("character");
-            if(cellAttr!=null&&cellAttr.length()>=12&&characterAttr!=null&&characterAttr.length()>=9) {
-                final Set<Entry<String, Object>> cellEntries = cellAttr.entrySet();
-                final Iterator<Entry<String, Object>> cellEntryIter = cellEntries.iterator();
-                while(cellEntryIter.hasNext()) {
-                    final Entry<String, Object> cellEntry = cellEntryIter.next();
-                    if(cellEntry.getValue()!=JSONObject.NULL) {
-                        return false;
-                    }
-                }
-                final Set<Entry<String, Object>> characterEntries = characterAttr.entrySet();
-                final Iterator<Entry<String, Object>> characterEntryIter = characterEntries.iterator();
-                while(characterEntryIter.hasNext()) {
-                    final Entry<String, Object> characterEntry = characterEntryIter.next();
-                    if(characterEntry.getValue()!=JSONObject.NULL) {
-                        return false;
-                    }
-                }
-                return true;
-            }
-        }
-        return false;
-    }
-
-    public void fillCellRange(int sheetIndex, JSONArray start, JSONArray optEnd, Object optValue, Object optResult, JSONObject optAttrs, int optShared, JSONArray optRef)
+    public void fillCellRange(int sheetIndex, JSONArray start, JSONArray optEnd, Object optValue, Object optResult, JSONObject optAttrs, AttrsMode attrsMode, int optShared, JSONArray optRef)
         throws Exception {
 
         if(start.length()!=2)
@@ -333,22 +309,15 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
             worksheet.setSheetData(sheetData);
         }
 
-        final boolean clearContent = optValue!=null&&optValue==JSONObject.NULL;
-        final boolean clearAttributes = isClearAttributes(optAttrs);
+        final boolean clearContent = optValue==JSONObject.NULL;
         final boolean setContent = ((optValue!=null)||(optResult!=null))&&!clearContent;
-        final boolean setAttributes = (optAttrs!=null)&&!clearAttributes;
-
-        // check if we have to create cells for the fillRange..
-        if(setContent||setAttributes) {
-
-        	Map<Long, Long> styleMapping = null;
+        if(attrsMode==AttrsMode.REPLACE) {
+            Map<Long, Long> styleMapping = new HashMap<Long, Long>();
             final Long cellSi = optRef!=null ? sheetData.getNextSi() : null;
             for(int y=start.getInt(1); y<=optEnd.getInt(1); y++) {
                 for(int x=start.getInt(0); x<=optEnd.getInt(0); x++) {
-
                     final Row row = sheetData.getRow(y, true);
                     final Cell cell = row.getCell(x, true);
-
                     if(setContent) {
                         if(optRef==null) {
                             setCellContent(cell, optValue, optResult, null, null);
@@ -358,18 +327,37 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
                             setCellContent(cell, optValue, optResult, cellRef, cellSi);
                         }
                     }
-                    if(setAttributes) {
-                    	if(styleMapping==null) {
-                    		styleMapping = new HashMap<Long, Long>();
-                    	}
-                    	CellUtils.applyCellProperties(operationDocument, worksheetPart, optAttrs, cell, getOperationDocument().getStylesheet(true), styleMapping);
+                    else if (clearContent) {
+                        setCellContent(cell, JSONObject.NULL, null, null, null);
                     }
+                    CellUtils.applyCellPropertiesToCell(operationDocument, worksheetPart, sheetData, optAttrs, attrsMode, row, cell, getOperationDocument().getStylesheet(true), styleMapping);
                 }
             }
         }
-        if(clearContent||clearAttributes) {
-            // only deleting cell content or cell attrs or both ... we are using another iterator
-            clearCellRange(sheetIndex, start, optEnd, clearContent, clearAttributes);
+        else if(setContent||optAttrs!=null) {
+        	Map<Long, Long> styleMapping = null;
+            final Long cellSi = optRef!=null ? sheetData.getNextSi() : null;
+            for(int y=start.getInt(1); y<=optEnd.getInt(1); y++) {
+                for(int x=start.getInt(0); x<=optEnd.getInt(0); x++) {
+                    final Row row = sheetData.getRow(y, true);
+                    final Cell cell = row.getCell(x, true);
+                    if(setContent||clearContent) {
+                        if(optRef==null) {
+                            setCellContent(cell, optValue, optResult, null, null);
+                        }
+                        else {
+                            final String cellRef = optRef.getInt(0)==x&&optRef.getInt(1)==y ? SmlUtils.getCellRefRange(start.getInt(0), start.getInt(1), optEnd.getInt(0), optEnd.getInt(1)) : null;
+                            setCellContent(cell, optValue, optResult, cellRef, cellSi);
+                        }
+                    }
+                    if(optAttrs!=null) {
+                    	if(styleMapping==null) {
+                    		styleMapping = new HashMap<Long, Long>();
+                    	}
+                    	CellUtils.applyCellPropertiesToCell(operationDocument, worksheetPart, sheetData, optAttrs, attrsMode, row, cell, getOperationDocument().getStylesheet(true), styleMapping);
+                    }
+                }
+            }
         }
     }
 
@@ -413,7 +401,22 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         }
     }
 
-    public void setRowAttributes(int sheetIndex, int start, int optEnd, JSONObject attrs)
+    private boolean nullOnlyAttributes(JSONObject attrs) {
+        final Iterator<Entry<String, Object>> iter = attrs.entrySet().iterator();
+        while(iter.hasNext()) {
+            final Entry<String, Object> entry = iter.next();
+            if(entry.getValue() instanceof JSONObject) {
+                if(nullOnlyAttributes((JSONObject)entry.getValue())==false) {
+                    return false;
+                }
+            }
+            else if(entry.getValue()!=JSONObject.NULL) {
+                return false;
+            }
+        }
+        return true;
+    }
+    public void setRowAttributes(int sheetIndex, int start, int optEnd, JSONObject attrs, AttrsMode attrsMode)
         throws Exception {
 
         if(optEnd==-1)
@@ -425,15 +428,20 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
             sheetData = Context.getsmlObjectFactory().createSheetData();
             worksheet.setSheetData(sheetData);
         }
-    	final Map<Long, Long> rowStyleCache = optEnd-start>0 ? new HashMap<Long, Long>() : null;
-    	final Map<Long, Long> rowCellStyleCache = new HashMap<Long, Long>();
+        final boolean nullOnly = nullOnlyAttributes(attrs);
+        final Map<Long, Long> rowStyleCache = optEnd-start>0 ? new HashMap<Long, Long>() : null;
+        final Map<Long, Long> rowCellStyleCache = new HashMap<Long, Long>();
         for(int y=start; y<=optEnd; y++) {
-            SheetUtils.applyRowProperties(getOperationDocument(), sheetData, start, attrs, rowCellStyleCache);
-            RowUtils.applyRowProperties(getOperationDocument(), attrs, sheetData.getRow(y, true), getOperationDocument().getStylesheet(true), rowStyleCache);
+            SheetUtils.applyRowProperties(getOperationDocument(), sheetData, y, attrs, attrsMode, rowCellStyleCache);
+            final Row row = sheetData.getRow(y, !nullOnly);
+            if(row!=null) {
+                RowUtils.applyRowProperties(getOperationDocument(), attrs, attrsMode, row, getOperationDocument().getStylesheet(true), rowStyleCache);
+                sheetData.applyCachedRowData(row);
+            }
         }
     }
 
-    public void setColumnAttributes(int sheetIndex, int start, int optEnd, JSONObject attrs)
+    public void setColumnAttributes(int sheetIndex, int start, int optEnd, JSONObject attrs, AttrsMode attrsMode)
         throws Exception {
 
         if(optEnd==-1)
@@ -449,7 +457,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         // entries have to be merged ...
         start++;
         optEnd++;
-        
+
     	final Map<Long, Long> columnStyleCache = optEnd-start>0 ? new HashMap<Long, Long>() : null;
     	final Map<Long, Long> columnCellStyleCache = new HashMap<Long, Long>();
         final Cols cols = colsList.get(0);
@@ -459,8 +467,8 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
                 return;
             if(col.getMax()<start)
                 continue;
-            SheetUtils.applyColumnProperties(getOperationDocument(), worksheet.getSheetData(), (int)(col.getMin()-1), (int)(col.getMax()-1), attrs, columnCellStyleCache);
-            ColumnUtils.applyColumnProperties(getOperationDocument(), attrs, col, worksheet, getOperationDocument().getStylesheet(true), columnStyleCache);
+            SheetUtils.applyColumnProperties(getOperationDocument(), worksheet.getSheetData(), (int)(col.getMin()-1), (int)(col.getMax()-1), attrs, attrsMode, columnCellStyleCache);
+            ColumnUtils.applyColumnProperties(getOperationDocument(), attrs, attrsMode, col, worksheet, getOperationDocument().getStylesheet(true), columnStyleCache);
         }
     }
 
@@ -614,6 +622,13 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
             }
         }
         handleInsertDeleteColRow(sheetIndex, start, count, ColRow.Column, 1);
+
+        // taking care that the inserted columns are not hidden
+        final JSONObject attrs = new JSONObject();
+        final JSONObject column = new JSONObject();
+        column.put("visible", JSONObject.NULL);
+        attrs.put("column", column);
+        setColumnAttributes(sheetIndex, start, optEnd, attrs, AttrsMode.EXTEND);
     }
 
     public void deleteColumns(int sheetIndex, int start, int optEnd) {
@@ -699,6 +714,28 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         handleInsertDeleteColRow(sheetIndex, start, count, ColRow.Column, -1);
     }
 
+    private SmlUtils.CellRefRange modifyCellRefRange(SmlUtils.CellRefRange cellRefRange, int start, int count,ColRow colrow, int rel, boolean append) {
+
+        SmlUtils.CellRefRange newCellRefRange = null;
+        if(rel>0) {
+            if(colrow==ColRow.Column) {
+                newCellRefRange = SmlUtils.CellRefRange.insertColumnRange(cellRefRange, start, count, append);
+            }
+            else {
+                newCellRefRange = SmlUtils.CellRefRange.insertRowRange(cellRefRange, start, count, append);
+            }
+        }
+        else {
+            if(colrow==ColRow.Column) {
+                newCellRefRange = SmlUtils.CellRefRange.deleteColumnRange(cellRefRange, start, (start+count)-1);
+            }
+            else {
+                newCellRefRange = SmlUtils.CellRefRange.deleteRowRange(cellRefRange, start, (start+count)-1);
+            }
+        }
+        return newCellRefRange;
+    }
+
     private void handleInsertDeleteColRow(int sheetIndex, int start, int count, ColRow colrow, int rel){
 
         final Workbook workbook = workbookPart.getJaxbElement();
@@ -715,47 +752,70 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
                 if(sheetRelationshipsPart!=null) {
                     final Worksheet worksheet = sheetPart.getJaxbElement();
 
-                    // check dataValidations
-                    final List<IDataValidations> validations = getOperationDocument().getDataValidations(worksheet);
-                    for(final IDataValidations dataValidations:validations) {
-                        final List<? extends IDataValidation> dataValidationList = dataValidations.getDataValidation();
-                        for(int j = dataValidationList.size()-1;j>=0;j--) {
-                            final IDataValidation dataValidation = dataValidationList.get(j);
-                            final List<String> sqrefList = dataValidation.getsqref();
-                            for(int k=sqrefList.size()-1; k>=0; k--) {
-                                final SmlUtils.CellRefRange cellRefRange = SmlUtils.createCellRefRange(sqrefList.get(k));
-                                if(cellRefRange!=null) {
-                                    SmlUtils.CellRefRange newCellRefRange = null;
-                                    if(rel>0) {
-                                        if(colrow==ColRow.Column) {
-                                            newCellRefRange = SmlUtils.CellRefRange.insertColumnRange(cellRefRange, start, count, true);
+                    if(sheetIndex==i) {
+
+                        // check dataValidations
+                        final List<IDataValidations> validations = OperationDocument.getDataValidations(worksheet);
+                        for(final IDataValidations dataValidations:validations) {
+                            final List<? extends IDataValidation> dataValidationList = dataValidations.getDataValidation();
+                            for(int j = dataValidationList.size()-1;j>=0;j--) {
+                                final IDataValidation dataValidation = dataValidationList.get(j);
+                                final List<String> sqrefList = dataValidation.getsqref();
+                                for(int k=sqrefList.size()-1; k>=0; k--) {
+                                    final SmlUtils.CellRefRange cellRefRange = SmlUtils.createCellRefRange(sqrefList.get(k));
+                                    if(cellRefRange!=null) {
+                                        final SmlUtils.CellRefRange newCellRefRange = modifyCellRefRange(cellRefRange, start, count, colrow, rel, true);
+                                        if(newCellRefRange==null) {
+                                            sqrefList.remove(k);
                                         }
-                                        else {
-                                            newCellRefRange = SmlUtils.CellRefRange.insertRowRange(cellRefRange, start, count, true);
+                                        else if(!newCellRefRange.equals(cellRefRange)) {
+                                            sqrefList.set(k, SmlUtils.getCellRefRange(newCellRefRange));
                                         }
                                     }
-                                    else {
-                                        if(colrow==ColRow.Column) {
-                                            newCellRefRange = SmlUtils.CellRefRange.deleteColumnRange(cellRefRange, start, (start+count)-1);
+                                }
+                                if(sqrefList.size()==0) {
+                                    dataValidationList.remove(j);
+                                }
+                            }
+                        }
+
+                        // check autoFilter
+                        final CTAutoFilter autoFilter = worksheet.getAutoFilter();
+                        if(autoFilter!=null) {
+                            final SmlUtils.CellRefRange cellRefRange = SmlUtils.createCellRefRange(autoFilter.getRef());
+                            if(cellRefRange!=null) {
+                                final SmlUtils.CellRefRange newCellRefRange = this.modifyCellRefRange(cellRefRange, start, count, colrow, rel, false);
+                                if(newCellRefRange==null) {
+                                    AutoFilterHelper.deleteTable(this, sheetIndex, null);
+                                }
+                                else if(!newCellRefRange.equals(cellRefRange)) {
+                                    autoFilter.setRef(SmlUtils.getCellRefRange(newCellRefRange));
+                                    AutoFilterHelper.insertDefinedTableName(this, Long.valueOf(sheetIndex), null, newCellRefRange);
+
+                                    // applying changed colIds
+                                    if(colrow==ColRow.Column) {
+                                        final List<CTFilterColumn> filterColumnList = autoFilter.getFilterColumn();
+                                        for(int j=0; j<filterColumnList.size(); j++) {
+                                            final CTFilterColumn filterColumn = filterColumnList.get(j);
+                                            final long colId = filterColumn.getColId();
+                                            if(rel>0&&colId>=start) {
+                                                filterColumn.setColId(colId+count);
+                                            }
+                                            else if(((colId>=start)&&(colId<(start+count)))) {
+                                                filterColumnList.remove(j--);
+                                            }
+                                            else if(colId>=start+count) {
+                                                filterColumn.setColId(colId-count);
+                                            }
                                         }
-                                        else {
-                                            newCellRefRange = SmlUtils.CellRefRange.deleteRowRange(cellRefRange, start, (start+count)-1);
-                                        }
-                                    }
-                                    if(newCellRefRange==null) {
-                                        sqrefList.remove(k);
-                                    }
-                                    else if(!newCellRefRange.equals(cellRefRange)) {
-                                        sqrefList.set(k, SmlUtils.getCellRefRange(newCellRefRange));
                                     }
                                 }
                             }
-                            if(sqrefList.size()==0) {
-                                dataValidationList.remove(j);
-                            }
                         }
                     }
 
+                    // for each worksheet -->
+
                     // check charts
                     final org.xlsx4j.sml.CTDrawing ctDrawing = worksheet.getDrawing();
                     if(ctDrawing!=null) {
@@ -877,7 +937,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
     }
 
     public void deleteSheet(int sheetIndex) {
-      	
+
         final Workbook workbook = workbookPart.getJaxbElement();
         final Sheets sheets = workbook.getSheets();
         final List<Sheet> sheetList = sheets.getSheet();
@@ -940,18 +1000,24 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 	    	final List<Relationship> sourceRelationshipList = sourceRelationships.getRelationship();
 	        for(int relIndex = 0; relIndex<sourceRelationshipList.size(); relIndex++) {
 	        	final Relationship sourceRelationship = sourceRelationshipList.get(relIndex);
+	        	final String sourceType = sourceRelationship.getType();
 	        	final String targetMode = sourceRelationship.getTargetMode();
-	        	if(targetMode==null||targetMode.equalsIgnoreCase("Internal")) {
-	        		if(type==null||sourceRelationship.getType().equals(type)) {
-						final Part part = sourceRelationshipsPart.getPart(sourceRelationship.getId());
-						if(part instanceof JaxbDmlPart) {
-							cloneDmlPart(destPart, (JaxbDmlPart<?>)part, sourceRelationship.getId());
-							continue;
-						}
-						else if(part instanceof DefaultXmlPart) {
-							cloneDefaultXmlPart(destPart, (DefaultXmlPart)part, sourceRelationship.getId());
-							continue;
-						}
+	        	if((targetMode==null||targetMode.equalsIgnoreCase("Internal"))&&!sourceType.equals(Namespaces.HYPERLINK)) {
+	        		if(type==null||sourceType.equals(type)) {
+	        		    try {
+    						final Part part = sourceRelationshipsPart.getPart(sourceRelationship.getId());
+    						if(part instanceof JaxbDmlPart) {
+    							cloneDmlPart(destPart, (JaxbDmlPart<?>)part, sourceRelationship.getId());
+    							continue;
+    						}
+    						else if(part instanceof DefaultXmlPart) {
+    							cloneDefaultXmlPart(destPart, (DefaultXmlPart)part, sourceRelationship.getId());
+    							continue;
+    						}
+	        		    }
+	        		    catch(InvalidFormatException e) {
+	        		        //
+	        		    }
 	        		}
 	        	}
         		destPart.getRelationshipsPart(true).getRelationships().getRelationship().add(XmlUtils.deepCopy(sourceRelationship, org.docx4j.jaxb.Context.getJcRelationships()));
@@ -961,7 +1027,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 
     private void cloneDmlPart(Part destPart, JaxbDmlPart<?> sourcePart, String relationshipId)
     	throws Docx4JException, InvalidFormatException, JAXBException, PartUnrecognisedException {
-    	
+
 		final JaxbDmlPart<?> clonedPart = (JaxbDmlPart<?>)JaxbDmlPart.newPartForContentType(sourcePart.getContentType(), sourcePart.getPartName().getName());
     	final ByteArrayOutputStream baos = new ByteArrayOutputStream();
     	sourcePart.marshal(baos);
@@ -972,7 +1038,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 
     private void cloneDefaultXmlPart(Part destPart, DefaultXmlPart sourcePart, String relationshipId)
     	throws Docx4JException, JAXBException {
-    	
+
 		final Document destDoc = (Document) sourcePart.getDocument().cloneNode(true);
 		final DefaultXmlPart clonedPart = new DefaultXmlPart(sourcePart.getPartName());
 		clonedPart.setContentType(new ContentType(sourcePart.getContentType()));
@@ -1041,7 +1107,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 	                }
 	            }
 	            // updating data validations
-	            final List<IDataValidations> validations = getOperationDocument().getDataValidations(worksheet);
+	            final List<IDataValidations> validations = OperationDocument.getDataValidations(worksheet);
 	            for(final IDataValidations dataValidations:validations) {
 	                for(final IDataValidation dataValidation:dataValidations.getDataValidation()) {
 	                    dataValidation.setFormula1(updateSheetName(dataValidation.getFormula1(), encodedOldName, encodedNewName));
@@ -1110,7 +1176,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 	        }
 	    }
     }
-    
+
     public static int getNameOffset(String content, int startOffset) {
 
         boolean stringLiteral = false;
@@ -1228,17 +1294,23 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 
     	if(attrs!=null) {
             final Workbook workbook = workbookPart.getJaxbElement();
+            final Worksheet worksheet = getWorksheet(sheetIndex);
             final JSONObject sheetProperties = attrs.optJSONObject("sheet");
             if(sheetProperties!=null) {
-            	final Worksheet worksheet = getWorksheet(sheetIndex);
                 SheetUtils.applySheetProperties(workbook, sheetIndex, worksheet, attrs.optJSONObject("sheet"));
-                SheetUtils.applyWorksheetColumnProperties(getOperationDocument(), worksheet, attrs.optJSONObject("column"));
-                SheetUtils.applyWorksheetRowProperties(getOperationDocument(), worksheet, attrs.optJSONObject("row"));
                 final boolean visible = sheetProperties.optBoolean("visible", true);
                 if(!visible) {
                     SheetUtils.updateBookView(workbook);
                 }
             }
+            final JSONObject columnProperties = attrs.optJSONObject("column");
+            if(columnProperties!=null) {
+                SheetUtils.applyWorksheetColumnProperties(getOperationDocument(), worksheet, columnProperties);
+            }
+            final JSONObject rowProperties = attrs.optJSONObject("row");
+            if(rowProperties!=null) {
+                SheetUtils.applyWorksheetRowProperties(getOperationDocument(), worksheet, rowProperties);
+            }
         }
     }
 
@@ -1300,7 +1372,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 
     public Long getStyleIndex(String styleId) {
         if(styleId==null||styleId.isEmpty()) {
-            return null;
+            return Long.valueOf(0);
         }
         return styleIdToIndex.get(styleId);
     }
@@ -1334,13 +1406,40 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
             styleMap.put(styleId, cellStyleXf);
             styleIdToIndex.put(styleId, ctXfIndex);
         }
+        final JSONObject cellApplyProperties = attrs.optJSONObject("apply");
+        if(cellApplyProperties!=null) {
+            final Object applyBorder = cellApplyProperties.opt("border");
+            if(applyBorder!=null) {
+                cellStyleXf.setApplyBorder(applyBorder instanceof Boolean&&!((Boolean)applyBorder).booleanValue()?false:null);
+            }
+            final Object applyAlign = cellApplyProperties.opt("align");
+            if(applyAlign!=null) {
+                cellStyleXf.setApplyAlignment(applyAlign instanceof Boolean&&!((Boolean)applyAlign).booleanValue()?false:null);
+            }
+            final Object applyNumber = cellApplyProperties.opt("number");
+            if(applyNumber!=null) {
+                cellStyleXf.setApplyNumberFormat(applyNumber instanceof Boolean&&!((Boolean)applyNumber).booleanValue()?false:null);
+            }
+            final Object applyProtect = cellApplyProperties.opt("protect");
+            if(applyProtect!=null) {
+                cellStyleXf.setApplyProtection(applyProtect instanceof Boolean&&!((Boolean)applyProtect).booleanValue()?false:null);
+            }
+            final Object applyFont = cellApplyProperties.opt("font");
+            if(applyFont!=null) {
+                cellStyleXf.setApplyProtection(applyFont instanceof Boolean&&!((Boolean)applyFont).booleanValue()?false:null);
+            }
+            final Object applyFill = cellApplyProperties.opt("fill");
+            if(applyFill!=null) {
+                cellStyleXf.setApplyFill(applyFill instanceof Boolean&&!((Boolean)applyFill).booleanValue()?false:null);
+            }
+        }
         final JSONObject cellProperties = attrs.optJSONObject("cell");
         if(cellProperties!=null) {
             CellUtils.applyCellProperties(operationDocument, cellProperties, cellStyleXf, stylesheet, null);
         }
         final JSONObject characterProperties = attrs.optJSONObject("character");
         if(characterProperties!=null) {
-            CellUtils.applyCharacterProperties(operationDocument, null, null, characterProperties, cellStyleXf, stylesheet, null);
+            CellUtils.applyCharacterProperties(operationDocument, null, null, null, characterProperties, cellStyleXf, stylesheet, null);
         }
     }
 
@@ -1372,7 +1471,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         if(drawing==null) { // -> drawing part needs to be created
             drawing = Context.getsmlObjectFactory().createCTDrawing();
             worksheet.setDrawing(drawing);
-            
+
             drawingPart = new Drawing(new PartName("/xl/drawings/drawing"+ Integer.toString(sheetNumber+1) + ".xml"));
             drawingPart.setJaxbElement(org.docx4j.jaxb.Context.getDmlSpreadsheetDrawingObjectFactory().createCTDrawing());
             final Relationship dmlRelationship = worksheetPart.addTargetPart(drawingPart, AddPartBehaviour.RENAME_IF_NAME_EXISTS);
@@ -1382,7 +1481,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
             drawingPart = (Drawing)worksheetPart.getRelationshipsPart().getPart(drawing.getId());
         }
         Drawings.insertDrawing(getOperationDocument(), start.getInt(1), drawingPart, type, attrs);
-        
+
         applyDrawingAttributes(start, attrs);
     }
 
@@ -1398,7 +1497,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         final org.docx4j.dml.spreadsheetdrawing.CTDrawing drawings = drawingsPart.getJaxbElement();
         final ICellAnchor cellAnchor = drawings.getEGAnchor().get(start.getInt(1));
         final HashMap<String, Object> drawingPropertyMap = DMLGraphic.getDrawingType(drawingsPart, cellAnchor);
-        
+
         final JSONObject chartProperties = attrs.optJSONObject("chart");
         if(chartProperties!=null) {
             final CTChartSpace chartSpace = (CTChartSpace)drawingPropertyMap.get("chartObject");
@@ -1429,7 +1528,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         DMLGraphic.getGraphicProperties(drawing, drawing.getJaxbElement().getEGAnchor().get(start.getInt(1)).getGraphicFrame().getGraphic(), drawingPropertyMap);
         DMLChartSpace.insertChartSpace((CTChartSpace)drawingPropertyMap.get("chartObject"), sheetName, series, attrs);
     }
-    
+
     public void setChartDataSeriesAttributes(JSONArray start, int series, JSONObject attrs)
             throws Exception {
 
@@ -1450,7 +1549,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         DMLChartSpace.deleteChartSpace((CTChartSpace)drawingPropertyMap.get("chartObject"), series);
     }
 
-    public void insertName(Long sheet, String name, String formula) {
+    public void insertName(Long sheet, String name, String formula, Boolean hidden) {
 
     	if(sheet<0) {
     		sheet = null;
@@ -1460,30 +1559,19 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         DefinedNames definedNames = workbook.getDefinedNames();
         if(definedNames==null) {
             definedNames = Context.getsmlObjectFactory().createDefinedNames();
+            definedNames.setParent(workbook);
             workbook.setDefinedNames(definedNames);
         }
-
-        int i;
-        CTDefinedName definedName = null;
-        final List<CTDefinedName> definedNameList = definedNames.getDefinedName();
-        for(i=0; i<definedNameList.size();i++) {
-            definedName = definedNameList.get(i);
-            // check if the name is already used..
-            if(definedName.getName().equals(name)) {
-                final Long localSheetId = definedName.getLocalSheetId();
-                if((sheet==null&&localSheetId==null)||(sheet!=null&&localSheetId!=null&&sheet.longValue()==localSheetId.longValue())) {
-                	break;
-                }
-            }
+        final HashMap<String, CTDefinedName> definedNameMap = definedNames.getDefinedNameMap();
+        final CTDefinedName definedName = Context.getsmlObjectFactory().createCTDefinedName();
+        definedName.setParent(definedNames);
+        definedName.setName(name);
+        definedName.setLocalSheetId(sheet);
+        definedName.setValue(formula);
+        if(hidden!=null) {
+            definedName.setHidden(hidden.booleanValue()?true:null);
         }
-        if(i==definedNameList.size()) {
-            definedName = Context.getsmlObjectFactory().createCTDefinedName();
-        	definedName.setParent(definedNames);
-            definedName.setName(name);
-            definedName.setLocalSheetId(sheet);
-            definedNameList.add(definedName);
-        }
-    	definedName.setValue(formula);
+        definedNameMap.put(DefinedNames.getDefinedNameKey(sheet, name), definedName);
     }
 
     public void deleteName(Long sheet, String name) {
@@ -1491,19 +1579,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         final Workbook workbook = workbookPart.getJaxbElement();
         final DefinedNames definedNames = workbook.getDefinedNames();
         if(definedNames!=null) {
-            final List<CTDefinedName> definedNameList = definedNames.getDefinedName();
-            for(int i=0; i<definedNameList.size();i++) {
-                final CTDefinedName definedName = definedNameList.get(i);
-
-                // check if the name is already used..
-                if(definedName.getName().equals(name)) {
-                    final Long localSheetId = definedName.getLocalSheetId();
-                    if((sheet==null&&localSheetId==null)||(sheet!=null&&localSheetId!=null&&sheet.longValue()==localSheetId.longValue())) {
-                        definedNameList.remove(i);
-                        return;
-                    }
-                }
-            }
+            definedNames.getDefinedNameMap().remove(DefinedNames.getDefinedNameKey(sheet, name));
         }
         log.debug("xlsx:deleteName ... could not find name within definedNameList");
     }
@@ -1540,10 +1616,10 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 	public void setChartLegendAttributes(JSONArray start, JSONObject attrs) throws Exception {
 		final CTChartSpace chartSpace = getChartSpace(start);
 		final CTChart chart = chartSpace.getChart();
-		
+
 		DMLChartSpace.setLegendFromAttrs(chart, attrs);
 	}
-	
+
 	private Map<String, Object> getDrawingProperties(JSONArray start) throws Exception {
 
 		final Drawing drawing = getDrawing(start.getInt(0));
@@ -1551,7 +1627,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
 		DMLGraphic.getGraphicProperties(drawing, drawing.getJaxbElement().getEGAnchor().get(start.getInt(1)).getGraphicFrame().getGraphic(), drawingPropertyMap);
 		return drawingPropertyMap;
 	}
-	
+
 	private CTChartSpace getChartSpace(JSONArray start) throws Exception {
 
 		final Map<String, Object> drawingPropertyMap = getDrawingProperties(start);
