@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -96,10 +96,12 @@ import org.xlsx4j.sml.Workbook;
 import org.xlsx4j.sml.Worksheet;
 
 import com.openexchange.log.LogFactory;
+import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.FilterException.ErrorCode;
 import com.openexchange.office.ooxml.tools.Commons;
 import com.openexchange.office.ooxml.xlsx.OperationDocument;
+import com.openexchange.office.ooxml.xlsx.tools.AutoFilterHelper;
 import com.openexchange.office.ooxml.xlsx.tools.CellUtils;
 import com.openexchange.office.ooxml.xlsx.tools.ColumnUtils;
 import com.openexchange.office.ooxml.xlsx.tools.Drawings;
@@ -154,14 +156,37 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
             // creating styles
             final CTCellStyleXfs cellStyleXfs = stylesheet.getCellStyleXfs();
             if(cellStyleXfs!=null) {
-                final List<CTXf> xfs = cellStyleXfs.getXf();
-                for(int i=0;i<xfs.size();i++) {
+                final List<CTXf> cellStyleXfList = cellStyleXfs.getXf();
+                for(int i=0;i<cellStyleXfList.size();i++) {
                     final String styleId = indexToStyleId.get(new Long(i));
                     if(styleId!=null) {
+                        final CTXf cellStyleXf = cellStyleXfList.get(i);
                         String styleName = styleId;
                         JSONObject jsonProperties = new JSONObject();
-                        Commons.mergeJsonObjectIfUsed("cell", jsonProperties, CellUtils.createCellProperties(xfs.get(i), stylesheet, true));
-                        Commons.mergeJsonObjectIfUsed("character", jsonProperties, CellUtils.createCharacterProperties(xfs.get(i), stylesheet, true));
+                        Commons.mergeJsonObjectIfUsed("cell", jsonProperties, CellUtils.createCellProperties(cellStyleXf, stylesheet, true));
+                        Commons.mergeJsonObjectIfUsed("character", jsonProperties, CellUtils.createCharacterProperties(cellStyleXf, stylesheet, true));
+                        final JSONObject jsonApplyProperties = new JSONObject(0);
+                        if(cellStyleXf.isApplyBorder()!=null&&!cellStyleXf.isApplyBorder().booleanValue()) {
+                            jsonApplyProperties.put("border", false);
+                        }
+                        if(cellStyleXf.isApplyAlignment()!=null&&!cellStyleXf.isApplyAlignment().booleanValue()) {
+                            jsonApplyProperties.put("align", false);
+                        }
+                        if(cellStyleXf.isApplyNumberFormat()!=null&&!cellStyleXf.isApplyNumberFormat().booleanValue()) {
+                            jsonApplyProperties.put("number", false);
+                        }
+                        if(cellStyleXf.isApplyProtection()!=null&&!cellStyleXf.isApplyProtection().booleanValue()) {
+                            jsonApplyProperties.put("protect", false);
+                        }
+                        if(cellStyleXf.isApplyFont()!=null&&!cellStyleXf.isApplyFont().booleanValue()) {
+                            jsonApplyProperties.put("font", false);
+                        }
+                        if(cellStyleXf.isApplyFill()!=null&&!cellStyleXf.isApplyFill().booleanValue()) {
+                            jsonApplyProperties.put("fill", false);
+                        }
+                        if(!jsonApplyProperties.isEmpty()) {
+                            jsonProperties.put("apply", jsonApplyProperties);
+                        }
                         JSONObject attrs = jsonProperties.length()>0?jsonProperties:null;
                         String parentId = null;
                         Boolean hidden = null;
@@ -185,7 +210,9 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                                 }
                             }
                         }
-                        operationsArray.put(createInsertStyleSheetOperation("cell", styleId, styleName, attrs, parentId, hidden, uiPriority, isDefault, false));
+                        if(hidden==null||!hidden.booleanValue()) {
+                        	operationsArray.put(createInsertStyleSheetOperation("cell", styleId, styleName, attrs, parentId, hidden, uiPriority, isDefault, false));
+                        }
                     }
                 }
             }
@@ -219,7 +246,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         }
     }
 
-    
+
     public void createOperations()
     	throws Exception {
 
@@ -235,16 +262,16 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 	    final List<Sheet> sheetList = sheets.getSheet();
 	    for(int sheetIndex = 0; sheetIndex < sheetList.size(); sheetIndex++) {
 	    	final Sheet sheet = sheetList.get(sheetIndex);
-	    	final Part part = getSheetPart(sheet);
-	    	final String sheetType = getSheetType(part);
+	    	final Part part = OperationDocument.getSheetPart(spreadsheetMLPackage, sheet);
+	    	final String sheetType = OperationDocument.getSheetType(part);
 	        createInsertSheetOperation(sheet, sheetType, part, sheetIndex);
 	        if(sheetType.equals("worksheet")) {
         		createWorksheetOperations((WorksheetPart)part, sheetIndex, false);
 	        }
 	    }
     }
-    
-    public int createPreviewOperations()
+
+    public int createPreviewOperations(DocumentProperties documentProperties)
     	throws Exception {
 
     	final WorkbookPart workbookPart = spreadsheetMLPackage.getWorkbookPart();
@@ -259,8 +286,8 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 	    final List<Sheet> sheetList = sheets.getSheet();
 	    for(int sheetIndex = 0; sheetIndex < sheetList.size(); sheetIndex++) {
 	    	final Sheet sheet = sheetList.get(sheetIndex);
-	    	final Part part = getSheetPart(sheet);
-	    	final String sheetType = getSheetType(part);
+	    	final Part part = OperationDocument.getSheetPart(spreadsheetMLPackage, sheet);
+	    	final String sheetType = OperationDocument.getSheetType(part);
 	        createInsertSheetOperation(sheet, sheetType, part, sheetIndex);
 	        if(sheetType.equals("worksheet")) {
 	        	if(activeSheetIndex==sheetIndex) {
@@ -268,6 +295,8 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 	        	}
 	        }
 	    }
+        documentProperties.put(DocumentProperties.PROP_SPREADSHEET_SHEET_COUNT, sheetList.size());
+        documentProperties.put(DocumentProperties.PROP_SPREADHSEET_ACTIVE_SHEET_INDEX, activeSheetIndex);
 	    return activeSheetIndex;
     }
 
@@ -281,8 +310,8 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 	    final List<Sheet> sheetList = sheets.getSheet();
 	    for(int sheetIndex = 0; sheetIndex < sheetList.size(); sheetIndex++) {
 	    	final Sheet sheet = sheetList.get(sheetIndex);
-	    	final Part part = getSheetPart(sheet);
-	    	final String sheetType = getSheetType(part);
+	    	final Part part = OperationDocument.getSheetPart(spreadsheetMLPackage, sheet);
+	    	final String sheetType = OperationDocument.getSheetType(part);
 	        if(sheetType.equals("worksheet")) {
 	        	if(activeSheetIndex!=sheetIndex) {
 	        		createWorksheetOperations((WorksheetPart)part, sheetIndex, false);
@@ -299,8 +328,8 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         final ServiceLookup services = getOperationDocument().getServiceLookup();
         final Session session = getOperationDocument().getSession();
 
-        int maxCellCount  =      ConfigurationHelper.getIntegerOfficeConfigurationValue(services, session, "//spreadsheet/maxCells", 500000);
-        int maxSheetCount =      ConfigurationHelper.getIntegerOfficeConfigurationValue(services, session, "//spreadsheet/maxSheets", 256);
+        final int maxCellCount  =      ConfigurationHelper.getIntegerOfficeConfigurationValue(services, session, "//spreadsheet/maxCells", 500000);
+        final int maxSheetCount =      ConfigurationHelper.getIntegerOfficeConfigurationValue(services, session, "//spreadsheet/maxSheets", 256);
 
         final Sheets sheets = workbook.getSheets();
         if(sheets!=null) {
@@ -311,6 +340,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                 }
             }
             if(maxCellCount>0) {
+                int cellCount = maxCellCount;
                 final RelationshipsPart relationshipPart = spreadsheetMLPackage.getWorkbookPart().getRelationshipsPart();
                 for(Sheet sheet:sheetList) {
                     final Part part = relationshipPart.getPart(sheet.getId());
@@ -321,8 +351,8 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                             final Iterator<Row> rowIterator = sheetData.createRowIterator();
                             while(rowIterator.hasNext()) {
                                 final Row row = rowIterator.next();
-                                maxCellCount -= row.getCellCount();
-                                if(maxCellCount<0) {
+                                cellCount -= row.getCellCount();
+                                if(cellCount<0) {
                                     throw new FilterException("", ErrorCode.COMPLEXITY_TOO_HIGH);
                                 }
                             }
@@ -366,18 +396,22 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         if(definedNames==null) {
             return;
         }
-        final List<CTDefinedName> definedNameList = definedNames.getDefinedName();
-        for(CTDefinedName definedName:definedNameList) {
+        final HashMap<String, CTDefinedName> definedNameMap = definedNames.getDefinedNameMap();
+        for(CTDefinedName definedName:definedNameMap.values()) {
             addInsertNameOperation(definedName.getLocalSheetId(), definedName.getName(), definedName.getValue()!=null ? definedName.getValue() : "");
         }
     }
 
-    private JSONObject getCellObject(Cell cell, boolean preview)
+    private JSONObject getCellObject(Row row, Cell cell, boolean preview)
         throws Exception {
 
     	Object value = null;
     	Object formula = null;
 
+    	STCellType cellType = cell.getT();
+    	if(cellType==null) {
+    		cellType = STCellType.N;
+    	}
         final JSONObject cellObject = new JSONObject();
         final CTCellFormula cellFormula = cell.getF();
         final boolean isFormula = cellFormula!=null&&(cellFormula.getT()==STCellFormulaType.NORMAL||cellFormula.getT()==STCellFormulaType.SHARED);
@@ -389,7 +423,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                     cellObject.put("shared", si+1);
                 }
                 if(hasFormulaText) {
-                    final SmlUtils.CellRef cellRef = new SmlUtils.CellRef(cell.getColumn(), ((Row)cell.getParent()).getRow());
+                    final SmlUtils.CellRef cellRef = new SmlUtils.CellRef(cell.getColumn(), row.getRow());
                     cellObject.put("ref", cellRef.getJSONArray());
                 }
             }
@@ -397,7 +431,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                 formula = new String("=" + cellFormula.getValue());
         }
         if(preview||!isFormula) {
-        	if (cell.getT()==STCellType.INLINE_STR) {
+        	if (cellType==STCellType.INLINE_STR) {
 	            CTRst is = cell.getIs();
 	            if(is!=null) {
 	                // TODO: CTRst can include formatted text runs, but we will support simple t first.
@@ -407,7 +441,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 	            }
 	        }
 	        else if(cell.getV()!=null) {
-	            if(cell.getT()==STCellType.N) {
+	            if(cellType==STCellType.N) {
 	                try {
 	                    value = new Double(Double.parseDouble(cell.getV()));
 	                }
@@ -415,13 +449,13 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 	                    // TODO: ups
 	                }
 	            }
-	            else if(cell.getT()==STCellType.S) {
+	            else if(cellType==STCellType.S) {
 	                String sharedString = getOperationDocument().getSharedString(cell.getV());
 	                if(sharedString!=null) {
 	                    value = new String(getOperationDocument().escapeString(sharedString));
 	                }
 	            }
-	            else if(cell.getT()==STCellType.B) {
+	            else if(cellType==STCellType.B) {
 	                if(cell.getV().equals("0")||cell.getV().toLowerCase().equals("false")) {
 	                	value = new Boolean(false);
 	                }
@@ -453,36 +487,6 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         return cellObject;
     }
 
-    private Part getSheetPart(final Sheet sheet) {
-        final RelationshipsPart relationshipPart = spreadsheetMLPackage.getWorkbookPart().getRelationshipsPart();
-        return relationshipPart.getPart(sheet.getId());
-    }
-
-    private String getSheetType(final Part part)
-    	throws FilterException {
-
-        String sheetType = null;
-        if(part.getRelationshipType().equals("http://schemas.openxmlformats.org/officeDocument/2006/relationships/worksheet")) {
-            sheetType = "worksheet";
-        }
-        else if(part.getRelationshipType().equals("http://schemas.microsoft.com/office/2006/relationships/xlMacrosheet")) {
-            sheetType = "macrosheet";
-        }
-        else if(part.getRelationshipType().equals("http://schemas.microsoft.com/office/2006/relationships/xlIntlMacrosheet")) {
-            sheetType = "macrosheet";
-        }
-        else if(part.getRelationshipType().equals("http://schemas.openxmlformats.org/officeDocument/2006/relationships/dialogsheet")) {
-            sheetType = "dialogsheet";
-        }
-        else if(part.getRelationshipType().equals("http://schemas.openxmlformats.org/officeDocument/2006/relationships/chartsheet")) {
-            sheetType = "chartsheet";
-        }
-        if(sheetType==null) {
-            throw new FilterException("xlsx import: unknown sheettype detected.", ErrorCode.CRITICAL_ERROR);
-        }
-        return sheetType;
-    }
-
     /*
      * if != null the topLeftCell parameter is filled with the current sheet settings
      */
@@ -514,6 +518,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         createRowOperations(worksheet, sheetIndex, preview);
 
         MergeCellHelper.createMergeCellOperations(operationsArray, sheetIndex, worksheet.getMergeCells());
+        AutoFilterHelper.createAutoFilterOperations(operationsArray, sheetIndex, worksheet.getAutoFilter());
 
         // hyperlinks
         final CTHyperlinks hyperlinks = worksheet.getHyperlinks();
@@ -543,14 +548,14 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                 if(worksheetRelationships!=null) {
                     final Part drawingsPart = worksheetRelationships.getPart(drawingId);
                     if(drawingsPart.getRelationshipType().equals("http://schemas.openxmlformats.org/officeDocument/2006/relationships/drawing")&&drawingsPart instanceof org.docx4j.openpackaging.parts.DrawingML.Drawing) {
-                        Drawings.createDrawingOperations(operationsArray, (org.docx4j.openpackaging.parts.DrawingML.Drawing)drawingsPart, sheetIndex);
+                        Drawings.createDrawingOperations(this.getOperationDocument(), operationsArray, (org.docx4j.openpackaging.parts.DrawingML.Drawing)drawingsPart, sheetIndex);
                     }
                 }
             }
         }
 
         // validations
-        final List<IDataValidations> validations = getOperationDocument().getDataValidations(worksheet);
+        final List<IDataValidations> validations = OperationDocument.getDataValidations(worksheet);
         for(IDataValidations dataValidations:validations) {
             for(IDataValidation dataValidation:dataValidations.getDataValidation()) {
                 createDataValidation(sheetIndex, dataValidation);
@@ -587,7 +592,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
     		return false;
     	if(source.getCm()!=dest.getCm())
     		return false;
-    	if(source.getS()!=dest.getS())
+    	if(source.getStyle()!=dest.getStyle())
     		return false;
     	if(source.getV()==null&&dest.getV()==null)
     		return true;
@@ -601,31 +606,12 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 
     private void createRowOperations(Worksheet worksheet, int sheetIndex, boolean preview)
         throws Exception {
-    	
+
     	final SheetData sheetData = worksheet.getSheetData();
     	if(sheetData==null) {
     		return;
     	}
 
-/*
-    	// about 169 bytes required for an empty row
-    	// now 144 bytes
-    	getOperationDocument().getFreeHeapSpace();
-    	for(int i=100000; i<600000; i++) {
-    		sheetData.getRow(i, true);
-    	}
-
-    	// about 139 bytes required for an empty cell
-    	// now 131 bytes
-    	getOperationDocument().getFreeHeapSpace();
-    	for(int j = 100000; j<101000; j++) {
-    		Row asd = sheetData.getRow(j, true);
-    		for(int k=0; k<500; k++) {
-    			Cell blah = asd.getCell(k, true);
-    		}
-    	}
-    	getOperationDocument().getFreeHeapSpace();
-*/
     	boolean sheetRowsHiddenByDefault = false;
     	final CTSheetFormatPr sheetFormatPr = worksheet.getSheetFormatPr();
     	if(sheetFormatPr!=null) {
@@ -716,7 +702,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
             	lastCell = cell;
              	lastCellX = cellRef.getColumn();
             	if(repeat==false) {
-	            	lastCellObject = getCellObject(cell, preview);
+	            	lastCellObject = getCellObject(row, cell, preview);
             	}
             }
             if(lastCellObject!=null) {
