@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,16 +55,16 @@ import java.util.Map;
 import org.json.JSONException;
 import org.json.JSONObject;
 import org.xlsx4j.sml.CTStylesheet;
-import org.xlsx4j.sml.CTXf;
 import org.xlsx4j.sml.Row;
 
+import com.openexchange.office.FilterException;
+import com.openexchange.office.ooxml.tools.AttrsMode;
 import com.openexchange.office.ooxml.xlsx.OperationDocument;
-import com.openexchange.office.ooxml.xlsx.operations.ApplyOperationHelper;
 
 public class RowUtils {
 
-    public static void applyRowProperties(OperationDocument operationDocument, JSONObject rowProperties, Row row, CTStylesheet stylesheet, Map<Long, Long> rowStyleCache)
-        throws  JSONException {
+    public static void applyRowProperties(OperationDocument operationDocument, JSONObject rowProperties, AttrsMode attrsMode, Row row, CTStylesheet stylesheet, Map<Long, Long> rowStyleCache)
+        throws FilterException, JSONException {
 
         // Row properties
         JSONObject jsonRowProperties = rowProperties.optJSONObject("row");
@@ -103,54 +103,16 @@ public class RowUtils {
                 }
             }
         }
-    	if(rowStyleCache!=null) {
-    		final Long newStyle = rowStyleCache.get(row.getS());
-    		if(newStyle!=null) {
-    			row.setS(newStyle);
-    			return;
-    		}
-    	}
-        Long newStyleIndex=null;
-        if(rowProperties.has("styleId")) {
-            final Object styleId = rowProperties.opt("styleId");
-            if(styleId instanceof String) {
-                final ApplyOperationHelper applyOperationHelper = operationDocument.getApplyOperationHelper();
-                if(applyOperationHelper!=null) {
-                    newStyleIndex = applyOperationHelper.getStyleIndex((String)styleId);
-                }
-            }
-            else {
-                newStyleIndex = 0L;
-            }
-        }
-
-        final CTXf detachedXf = CellUtils.getDetachedXf(stylesheet, row.getS(), newStyleIndex);
-        final CTXf cellStyleXf = stylesheet.getCellStyleXfsByIndex(detachedXf.getXfId());
-
-        final JSONObject characterProperties = rowProperties.optJSONObject("character");
-        if (characterProperties != null) {
-            CellUtils.applyCharacterProperties(operationDocument, null, null, characterProperties, detachedXf, stylesheet, cellStyleXf);
-        }
-        final JSONObject cellProperties = rowProperties.optJSONObject("cell");
-        if(cellProperties!=null) {
-            CellUtils.applyCellProperties(operationDocument, cellProperties, detachedXf, stylesheet, cellStyleXf);
-        }
-        long index = stylesheet.getOrApplyCellXf(detachedXf);
-        if(index>=0) {
-            if(rowStyleCache!=null) {
-            	rowStyleCache.put(row.getS(), index);
-            }
-            row.setS(index);
-        }
+        row.setS(Long.valueOf(CellUtils.applyAutoStyle(operationDocument, row.getStyle(), rowProperties, attrsMode, stylesheet, rowStyleCache, null, null, null)));
         return;
     }
-    
+
     public static double map100thmmToPoint(double rowHeight) {
         return Math.round(rowHeight*72/25.40)/100d;
     }
 
     public static boolean compareRowProperties(Row r1, Row r2) {
-        if(r1.getS()!=r2.getS()) {
+        if(r1.getStyle()!=r2.getStyle()) {
             return false;
         }
         if(r1.getHt()!=null&&r2.getHt()!=null) {
@@ -195,7 +157,7 @@ public class RowUtils {
         if(rowProperties.length()>0) {
             attrs.put("row", rowProperties);
         }
-        long s = row.getS();
+        long s = row.getStyle();
         if(s>0) {
             attrs.put("styleId", "a" + Long.toString(s));
         }
@@ -205,7 +167,7 @@ public class RowUtils {
         return attrs;
     }
 
-    public static double mapPointTo100thmm(double rowHeight) {
-        return Math.round(rowHeight*2540/0.72+0.5)/100d;
+    public static long mapPointTo100thmm(double rowHeight) {
+        return Math.round(rowHeight*(2540d/72d));
     }
 }
