@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -79,6 +79,7 @@ import org.xlsx4j.sml.SmlUtils;
 import org.xlsx4j.sml.Workbook;
 import org.xlsx4j.sml.Worksheet;
 
+import com.openexchange.office.ooxml.tools.AttrsMode;
 import com.openexchange.office.ooxml.tools.Commons;
 import com.openexchange.office.ooxml.xlsx.OperationDocument;
 
@@ -321,10 +322,10 @@ public class SheetUtils {
     				sel.clear();
     				sel.add(SmlUtils.getCellRef(activeCellRef));
     			}
-	        }  
+	        }
         }
     }
-			
+
 	private static CTSelection getSelection(SheetView sheetView, boolean forceCreate) {
 
 		final STPane activePane = sheetView.getPane()!=null?sheetView.getPane().getActivePane():STPane.TOP_LEFT;
@@ -358,9 +359,9 @@ public class SheetUtils {
     	}
     	return sheetViewList.get(0);
     }
-    
+
     private static CTPane getOrCreatePane(Worksheet worksheet) {
-    	
+
     	final SheetView sheetView = getOrCreateSheetView(worksheet);
     	CTPane pane = sheetView.getPane();
     	if(pane==null) {
@@ -371,39 +372,40 @@ public class SheetUtils {
     	return pane;
     }
 
+    // check if the first visible sheet matches the bookView and removes each tabSelected from sheet
     public static void updateBookView(Workbook workbook) {
 
-        // check if the first visible sheet matches to the bookView
         final List<Sheet> sheetList = workbook.getSheets().getSheet();
-        int firstVisible = 0;
-        for(Sheet sh:sheetList) {
-            if(sh.getState()==STSheetState.VISIBLE) {
-                break;
+        Long firstVisible = null;
+        for(int i=0; i<sheetList.size(); i++) {
+            final Sheet sh = sheetList.get(i);
+            if(firstVisible==null&&sh.getState()==STSheetState.VISIBLE) {
+                firstVisible = Long.valueOf(i);
             }
-            firstVisible++;
         }
-        if(firstVisible>=sheetList.size()) {
-            firstVisible = 0;
+        if(firstVisible==null) {
+            firstVisible = Long.valueOf(0);
         }
         BookViews bookViews = workbook.getBookViews();
-        if(firstVisible>0||bookViews!=null) {
-            if(bookViews==null) {
-                bookViews = Context.getsmlObjectFactory().createBookViews();
-                workbook.setBookViews(bookViews);
+        if(bookViews==null) {
+            bookViews = Context.getsmlObjectFactory().createBookViews();
+            bookViews.setParent(workbook);
+            workbook.setBookViews(bookViews);
+        }
+        final List<CTBookView> workbookViewList = bookViews.getWorkbookView();
+        if(workbookViewList.isEmpty()) {
+            final CTBookView bookView = Context.getsmlObjectFactory().createCTBookView();
+            bookView.setParent(bookViews);
+            workbookViewList.add(bookView);
+        }
+        for(CTBookView bookView:workbookViewList) {
+            final int activeTab = (int)bookView.getActiveTab();
+            final int firstView = (int)bookView.getFirstSheet();
+            if(activeTab<0||activeTab>=sheetList.size()||sheetList.get(activeTab).getState()!=STSheetState.VISIBLE) {
+                bookView.setActiveTab(firstVisible);
             }
-            final List<CTBookView> workbookViewList = bookViews.getWorkbookView();
-            if(workbookViewList.isEmpty()) {
-                workbookViewList.add(Context.getsmlObjectFactory().createCTBookView());
-            }
-            for(CTBookView bookView:workbookViewList) {
-                final int activeTab = (int)bookView.getActiveTab();
-                final int firstView = (int)bookView.getFirstSheet();
-                if(activeTab<0||activeTab>=sheetList.size()||sheetList.get(activeTab).getState()!=STSheetState.VISIBLE) {
-                    bookView.setActiveTab(new Long(firstVisible));
-                }
-                if(firstView<0||firstView>=sheetList.size()||sheetList.get(firstView).getState()!=STSheetState.VISIBLE) {
-                    bookView.setFirstSheet(new Long(firstVisible));
-                }
+            if(firstView<0||firstView>=sheetList.size()||sheetList.get(firstView).getState()!=STSheetState.VISIBLE) {
+                bookView.setFirstSheet(firstVisible);
             }
         }
     }
@@ -557,10 +559,10 @@ public class SheetUtils {
     	}
     	return sheetFormatPr;
     }
-    
+
     public static void applyWorksheetColumnProperties(OperationDocument operationDocument, Worksheet worksheet, JSONObject columnProperties)
             throws JSONException {
-    	
+
     	if(columnProperties==null)
     		return;
     	final CTSheetFormatPr sheetFormatPr = getOrCreateSheetFormatPr(worksheet);
@@ -568,7 +570,7 @@ public class SheetUtils {
         if(width!=null) {
         	sheetFormatPr.setDefaultColWidth(ColumnUtils.columnWidthToCharacter(operationDocument, width));
         }
-/*        
+/*
         final Object customWidth = columnProperties.opt("customWidth");
         if(customWidth!=null) {
         }
@@ -597,10 +599,10 @@ public class SheetUtils {
         	}
         }
     }
-    
+
     public static void applyWorksheetRowProperties(OperationDocument operationDocument, Worksheet worksheet, JSONObject rowProperties)
             throws JSONException {
-    	
+
     	if(rowProperties==null)
     		return;
 
@@ -633,7 +635,7 @@ public class SheetUtils {
     		}
     	}
     }
-    
+
     public static JSONObject createWorksheetColumnProperties(OperationDocument operationDocument, Worksheet worksheet)
         throws JSONException {
 
@@ -657,10 +659,14 @@ public class SheetUtils {
 
         final JSONObject rowProperties = new JSONObject();
         final CTSheetFormatPr sheetFormatPr = worksheet.getSheetFormatPr();
-        if(sheetFormatPr!=null) {
-        	rowProperties.put("height", RowUtils.mapPointTo100thmm(sheetFormatPr.getDefaultRowHeight()));
+
+        final boolean customHeight = sheetFormatPr!=null&&sheetFormatPr.isCustomHeight();
+        if(customHeight) {
+        	rowProperties.put("customHeight", true);
         }
-        rowProperties.put("customHeight", sheetFormatPr!=null?sheetFormatPr.isCustomHeight():false);
+        final double defaultRowHeight = customHeight?sheetFormatPr.getDefaultRowHeight():15;
+    	rowProperties.put("height", RowUtils.mapPointTo100thmm(defaultRowHeight));
+
         if(sheetFormatPr!=null&&sheetFormatPr.isZeroHeight()) {
         	rowProperties.put("visible", false);
         }
@@ -671,7 +677,7 @@ public class SheetUtils {
      * applying attrs to each physical available cell within the row
      *
      */
-    public static void applyRowProperties(OperationDocument operationDocument, SheetData sheetData, int start, JSONObject attrs, Map<Long, Long> rowCellStyleCache)
+    public static void applyRowProperties(OperationDocument operationDocument, SheetData sheetData, int start, JSONObject attrs, AttrsMode attrsMode, Map<Long, Long> rowCellStyleCache)
         throws Exception {
 
         Row row = sheetData.getRow(start, false);
@@ -680,7 +686,7 @@ public class SheetUtils {
         }
         final Iterator<Cell> cellIterator = row.createCellIterator();
         while(cellIterator.hasNext()) {
-            CellUtils.applyCellProperties(operationDocument, null, attrs, cellIterator.next(), operationDocument.getStylesheet(true), rowCellStyleCache);
+            CellUtils.applyCellPropertiesToCell(operationDocument, null, sheetData, attrs, attrsMode, row, cellIterator.next(), operationDocument.getStylesheet(true), rowCellStyleCache);
         }
     }
 
@@ -688,7 +694,7 @@ public class SheetUtils {
      * applying attrs to each physical available cell within the column
      *
      */
-    public static void applyColumnProperties(OperationDocument operationDocument, SheetData sheetData, int start, int end, JSONObject attrs, Map<Long, Long> columnCellStyleCache)
+    public static void applyColumnProperties(OperationDocument operationDocument, SheetData sheetData, int start, int end, JSONObject attrs, AttrsMode attrsMode, Map<Long, Long> columnCellStyleCache)
         throws Exception {
 
         final Iterator<Row> rowIterator = sheetData.createRowIterator();
@@ -697,7 +703,7 @@ public class SheetUtils {
             for(int i=start; i<=end; i++) {
                 final Cell cell = row.getCell(i, false);
                 if(cell!=null) {
-                    CellUtils.applyCellProperties(operationDocument, null, attrs, cell, operationDocument.getStylesheet(true), columnCellStyleCache);
+                    CellUtils.applyCellPropertiesToCell(operationDocument, null, sheetData, attrs, attrsMode, row, cell, operationDocument.getStylesheet(true), columnCellStyleCache);
                 }
             }
         }
