@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,6 +50,7 @@
 package com.openexchange.office.ooxml.xlsx.tools;
 
 import java.util.List;
+
 import org.docx4j.XmlUtils;
 import org.docx4j.dml.BaseStyles;
 import org.docx4j.dml.CTColorScheme;
@@ -75,6 +76,8 @@ import org.xlsx4j.sml.Cols;
 import org.xlsx4j.sml.STBorderStyle;
 import org.xlsx4j.sml.STHorizontalAlignment;
 import org.xlsx4j.sml.STVerticalAlignment;
+
+import com.openexchange.office.FilterException;
 import com.openexchange.office.ooxml.tools.Commons;
 import com.openexchange.office.ooxml.xlsx.OperationDocument;
 
@@ -317,7 +320,7 @@ public class Utils {
     public static double hmmToPixel(int hmm) {
     	return hmm * 96.0 / 2540.0;
     }
-    
+
     public static int HAIR_WIDTH_HMM= pixelToHmm(0.5);
     public static int THIN_WIDTH_HMM = pixelToHmm(1);
     public static int MEDIUM_WIDTH_HMM = pixelToHmm(2);
@@ -518,7 +521,7 @@ public class Utils {
     }
 
     public static void applyColor(OperationDocument operationDocument, CTColor color, JSONObject jsonColor)
-        throws JSONException {
+        throws FilterException, JSONException {
 
         if(color!=null&&jsonColor.has("type")) {
             String type = jsonColor.getString("type");
