@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -57,8 +57,10 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+
 import javax.xml.bind.DatatypeConverter;
 import javax.xml.bind.JAXBElement;
+
 import org.apache.commons.io.FilenameUtils;
 import org.apache.commons.logging.Log;
 import org.docx4j.dml.CTColor;
@@ -76,6 +78,7 @@ import org.docx4j.relationships.Relationship;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+
 import com.openexchange.log.LogFactory;
 import com.openexchange.office.ooxml.OperationDocument;
 import com.openexchange.office.tools.FileHelper;
@@ -107,7 +110,7 @@ final public class Commons {
             ret = fillString.substring( ret.length() ) + ret;
         return ret;
     }
-    
+
     public static String intToHexString(int color) {
         String ret = Integer.toHexString(color);
         if(ret.length() < 6)
@@ -315,7 +318,7 @@ final public class Commons {
 
         final double max = Math.max(Math.max(r, g), b);
         final double min = Math.min(Math.min(r, g), b);
-    	
+
         double h, s, l = (max + min) / 2;
         if(max==min) {
         	h = s = 0;
@@ -374,7 +377,7 @@ final public class Commons {
 
         	final double nVal1 = l <= 0.5 ? l * (1.0 + s) : l + s - l * s;
 	        final double nVal2 = 2.0 * l - nVal1;
-	
+
 	        r = hsl2rgbHelper(nVal2, nVal1, h + 120.0);
 	        g = hsl2rgbHelper(nVal2, nVal1, h);
 	        b = hsl2rgbHelper(nVal2, nVal1, h - 120.0);
@@ -561,7 +564,7 @@ final public class Commons {
        return null;
    }
 
-   public static Relationship createAndApplyGraphic(OperationDocument operationDocument, Part part, String imageUrl)
+   public static Relationship createGraphicRelation(OperationDocument operationDocument, Part part, String imageUrl)
        throws InvalidFormatException, PartUnrecognisedException {
 
        // check if the image is already part of the document
