@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -192,9 +192,12 @@ public class ConnectionComponent extends AbstractRealtimeJanitor implements Comp
             try {
                 Date currentDate = new Date();
                 Connection connection = connections.nextElement();
-                // don't try to save if we in a middle of a save or we were not able
-                // to save the document correctly.
-                if ((connection != null) && !connection.savingDocumentInProgress() && (connection.getLastFailSafeSaveError().getCode() == ErrorCode.NO_ERROR.getCode())) {
+
+                // Don't try to save, if the connection has already been disposed, we are
+                // in a middle of a save or we were not able to save the document correctly.
+                if ((connection != null) && !connection.isDisposed() &&
+                    !connection.savingDocumentInProgress() &&
+                    (connection.getLastFailSafeSaveError().getCode() == ErrorCode.NO_ERROR.getCode())) {
                     long diff = getTimeDifference(connection.getSaveTimeStamp(), currentDate);
                     long ops = connection.getNumberOfPendingOperations();
 
