@@ -3,6 +3,8 @@ package com.openexchange.office.tools.htmldoc;
 import java.util.List;
 import java.util.concurrent.atomic.AtomicInteger;
 
+import org.apache.commons.lang.StringEscapeUtils;
+
 import org.json.JSONArray;
 import org.json.JSONObject;
 
@@ -61,6 +63,15 @@ public class Drawing
         JSONObject assignedAttrs)
         throws Exception
     {
+        if (assignedAttrs != null && assignedAttrs.has("image")) {
+            JSONObject drawing = assignedAttrs.getJSONObject("image");
+            String imageUrl = drawing.optString("imageUrl", "");
+            if (imageUrl.contains("://")) {
+                imageUrl = imageUrl.replaceAll("_REPLACE_SESSION_ID_", "_REPLACESESSIONID_"); // avoiding session ID replacement on client side for remote URLs (54403)
+                drawing.put("imageUrl", imageUrl);
+            }
+        }
+
         super.setAttribute(start, end, assignedAttrs);
 
         // type must be checked now, because we support more than image
@@ -122,7 +133,11 @@ public class Drawing
             document.append(DRAWING_CONTENT);
 
             String image = CONTENT_IMAGE_START;
-            image += src;
+
+            String srcStr = StringEscapeUtils.escapeHtml(src);
+            srcStr = srcStr.replace("'", "&#39;"); // escaping all single quotes (49847)
+
+            image += srcStr;
             image += CONTENT_IMAGE_END;
 
             document.append(image);
